from PySide6.QtWidgets import  QDialog, QFileDialog
from debyetools.tpropsgui.ui_dialogSUMMARY import Ui_Dialog as Ui_SUMMARY
from debyetools.tpropsgui.atomtools import atomic_mass
from debyetools.aux_functions import load_V_E as dt_load_V_E

def log_mean(lst):
    import numpy as np
    l = len(lst)
    lm = np.sum([np.log(li)/l for li in lst])
    m = np.exp(lm)
    return m
class dialogSUMMARY(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_SUMMARY()
        self.ui.setupUi(self)

        self.ui.browsesummary.clicked.connect(self.getfilessummary)
        self.ui.browseposcar.clicked.connect(self.getfilesposcar)
        self.ui.ok.clicked.connect(self.on_pushButton_OK)
        self.filepath='.'

    def getfilessummary(self):
        summarypath, _ = QFileDialog.getOpenFileName(self, caption='Select a SUMMARY file')
        self.ui.summarypath.setText(summarypath)
        self.summarypath = summarypath

    def getfilesposcar(self):
        poscarpath, _ = QFileDialog.getOpenFileName(self, caption='Select a POSCAR or CONTCAR file')
        self.ui.poscarpath.setText(poscarpath)
        self.poscarpath = poscarpath

    def on_pushButton_OK(self):
        self.summarypath = self.ui.summarypath.text()
        self.poscarpath = self.ui.poscarpath.text()
        self.close()

    def closeEvent(self, event):
        V, E = dt_load_V_E(self.summarypath, self.poscarpath)
        with open(self.poscarpath) as f:
            poscar_lines = f.readlines()
        elmnts = poscar_lines[5].split()
        mult = poscar_lines[6].split()
        elmntsall = []
        for e, m in zip(elmnts, mult):
            elmntsall = elmntsall + [e]*int(m)
        masses = [float(atomic_mass[e])/100. for e in elmntsall]
#        print(elmntsall, masses, log_mean(masses))
        txt2paste = '#V\tE\n'
        for v, e in zip(V, E):
            txt2paste = txt2paste + '%.6e\t%.6e'%(v, e)+'\n'
        self.EvVtext.setPlainText(txt2paste)
        self.mass.setText(str(log_mean(masses)))
#        print('event', event)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'interatomic_params.ui'
##
## Created by: Qt User Interface Compiler version 6.4.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QHBoxLayout, QHeaderView,
    QLabel, QLineEdit, QPlainTextEdit, QPushButton,
    QSizePolicy, QTableWidget, QTableWidgetItem, QWidget)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(796, 395)
        self.horizontalLayoutWidget = QWidget(Form)
        self.horizontalLayoutWidget.setObjectName(u"horizontalLayoutWidget")
        self.horizontalLayoutWidget.setGeometry(QRect(280, 10, 241, 241))
        self.horizontalLayout = QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.label_3 = QLabel(Form)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setGeometry(QRect(10, 290, 49, 21))
        self.label_4 = QLabel(Form)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setGeometry(QRect(10, 320, 161, 21))
        self.lineEdit = QLineEdit(Form)
        self.lineEdit.setObjectName(u"lineEdit")
        self.lineEdit.setGeometry(QRect(140, 290, 91, 24))
        self.lineEdit_2 = QLineEdit(Form)
        self.lineEdit_2.setObjectName(u"lineEdit_2")
        self.lineEdit_2.setGeometry(QRect(140, 320, 91, 24))
        self.plainTextEdit = QPlainTextEdit(Form)
        self.plainTextEdit.setObjectName(u"plainTextEdit")
        self.plainTextEdit.setEnabled(True)
        self.plainTextEdit.setGeometry(QRect(530, 10, 261, 241))
        palette = QPalette()
        self.plainTextEdit.setPalette(palette)
        font = QFont()
        font.setFamilies([u"Cascadia Mono"])
        font.setPointSize(8)
        self.plainTextEdit.setFont(font)
        self.plainTextEdit.setFrameShape(QFrame.Box)
        self.plainTextEdit.setTabChangesFocus(False)
        self.plainTextEdit.setReadOnly(True)
        self.plainTextEdit.setBackgroundVisible(False)
        self.plainTextEdit.setCenterOnScroll(False)
        self.OKbutton = QPushButton(Form)
        self.OKbutton.setObjectName(u"OKbutton")
        self.OKbutton.setGeometry(QRect(680, 340, 80, 24))
        self.pushButton = QPushButton(Form)
        self.pushButton.setObjectName(u"pushButton")
        self.pushButton.setGeometry(QRect(590, 340, 71, 24))
        self.label = QLabel(Form)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 10, 31, 16))
        self.tableWidget = QTableWidget(Form)
        if (self.tableWidget.columnCount() < 3):
            self.tableWidget.setColumnCount(3)
        __qtablewidgetitem = QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, __qtablewidgetitem)
        __qtablewidgetitem1 = QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, __qtablewidgetitem1)
        __qtablewidgetitem2 = QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, __qtablewidgetitem2)
        if (self.tableWidget.rowCount() < 3):
            self.tableWidget.setRowCount(3)
        __qtablewidgetitem3 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(0, __qtablewidgetitem3)
        __qtablewidgetitem4 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(1, __qtablewidgetitem4)
        __qtablewidgetitem5 = QTableWidgetItem()
        self.tableWidget.setVerticalHeaderItem(2, __qtablewidgetitem5)
        __qtablewidgetitem6 = QTableWidgetItem()
        self.tableWidget.setItem(0, 0, __qtablewidgetitem6)
        __qtablewidgetitem7 = QTableWidgetItem()
        self.tableWidget.setItem(0, 1, __qtablewidgetitem7)
        __qtablewidgetitem8 = QTableWidgetItem()
        self.tableWidget.setItem(0, 2, __qtablewidgetitem8)
        __qtablewidgetitem9 = QTableWidgetItem()
        self.tableWidget.setItem(1, 0, __qtablewidgetitem9)
        __qtablewidgetitem10 = QTableWidgetItem()
        self.tableWidget.setItem(1, 1, __qtablewidgetitem10)
        __qtablewidgetitem11 = QTableWidgetItem()
        self.tableWidget.setItem(1, 2, __qtablewidgetitem11)
        __qtablewidgetitem12 = QTableWidgetItem()
        self.tableWidget.setItem(2, 0, __qtablewidgetitem12)
        __qtablewidgetitem13 = QTableWidgetItem()
        self.tableWidget.setItem(2, 1, __qtablewidgetitem13)
        __qtablewidgetitem14 = QTableWidgetItem()
        self.tableWidget.setItem(2, 2, __qtablewidgetitem14)
        self.tableWidget.setObjectName(u"tableWidget")
        self.tableWidget.setGeometry(QRect(10, 30, 171, 81))
        self.tableWidget.setFrameShape(QFrame.Box)
        self.tableWidget.horizontalHeader().setVisible(False)
        self.tableWidget.horizontalHeader().setDefaultSectionSize(50)
        self.tableWidget.verticalHeader().setDefaultSectionSize(25)
        self.tableWidget_2 = QTableWidget(Form)
        if (self.tableWidget_2.columnCount() < 4):
            self.tableWidget_2.setColumnCount(4)
        __qtablewidgetitem15 = QTableWidgetItem()
        self.tableWidget_2.setHorizontalHeaderItem(0, __qtablewidgetitem15)
        __qtablewidgetitem16 = QTableWidgetItem()
        self.tableWidget_2.setHorizontalHeaderItem(1, __qtablewidgetitem16)
        __qtablewidgetitem17 = QTableWidgetItem()
        self.tableWidget_2.setHorizontalHeaderItem(2, __qtablewidgetitem17)
        __qtablewidgetitem18 = QTableWidgetItem()
        self.tableWidget_2.setHorizontalHeaderItem(3, __qtablewidgetitem18)
        if (self.tableWidget_2.rowCount() < 10):
            self.tableWidget_2.setRowCount(10)
        __qtablewidgetitem19 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(0, __qtablewidgetitem19)
        __qtablewidgetitem20 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(1, __qtablewidgetitem20)
        __qtablewidgetitem21 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(2, __qtablewidgetitem21)
        __qtablewidgetitem22 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(3, __qtablewidgetitem22)
        __qtablewidgetitem23 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(4, __qtablewidgetitem23)
        __qtablewidgetitem24 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(5, __qtablewidgetitem24)
        __qtablewidgetitem25 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(6, __qtablewidgetitem25)
        __qtablewidgetitem26 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(7, __qtablewidgetitem26)
        __qtablewidgetitem27 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(8, __qtablewidgetitem27)
        __qtablewidgetitem28 = QTableWidgetItem()
        self.tableWidget_2.setVerticalHeaderItem(9, __qtablewidgetitem28)
        __qtablewidgetitem29 = QTableWidgetItem()
        self.tableWidget_2.setItem(0, 0, __qtablewidgetitem29)
        __qtablewidgetitem30 = QTableWidgetItem()
        self.tableWidget_2.setItem(0, 1, __qtablewidgetitem30)
        __qtablewidgetitem31 = QTableWidgetItem()
        self.tableWidget_2.setItem(0, 2, __qtablewidgetitem31)
        __qtablewidgetitem32 = QTableWidgetItem()
        self.tableWidget_2.setItem(0, 3, __qtablewidgetitem32)
        __qtablewidgetitem33 = QTableWidgetItem()
        self.tableWidget_2.setItem(1, 0, __qtablewidgetitem33)
        __qtablewidgetitem34 = QTableWidgetItem()
        self.tableWidget_2.setItem(1, 1, __qtablewidgetitem34)
        __qtablewidgetitem35 = QTableWidgetItem()
        self.tableWidget_2.setItem(1, 2, __qtablewidgetitem35)
        __qtablewidgetitem36 = QTableWidgetItem()
        self.tableWidget_2.setItem(1, 3, __qtablewidgetitem36)
        __qtablewidgetitem37 = QTableWidgetItem()
        self.tableWidget_2.setItem(2, 0, __qtablewidgetitem37)
        __qtablewidgetitem38 = QTableWidgetItem()
        self.tableWidget_2.setItem(2, 1, __qtablewidgetitem38)
        __qtablewidgetitem39 = QTableWidgetItem()
        self.tableWidget_2.setItem(2, 2, __qtablewidgetitem39)
        __qtablewidgetitem40 = QTableWidgetItem()
        self.tableWidget_2.setItem(2, 3, __qtablewidgetitem40)
        __qtablewidgetitem41 = QTableWidgetItem()
        self.tableWidget_2.setItem(3, 0, __qtablewidgetitem41)
        __qtablewidgetitem42 = QTableWidgetItem()
        self.tableWidget_2.setItem(3, 1, __qtablewidgetitem42)
        __qtablewidgetitem43 = QTableWidgetItem()
        self.tableWidget_2.setItem(3, 2, __qtablewidgetitem43)
        __qtablewidgetitem44 = QTableWidgetItem()
        self.tableWidget_2.setItem(3, 3, __qtablewidgetitem44)
        self.tableWidget_2.setObjectName(u"tableWidget_2")
        self.tableWidget_2.setGeometry(QRect(10, 130, 231, 131))
        self.tableWidget_2.setFrameShape(QFrame.Box)
        self.tableWidget_2.horizontalHeader().setDefaultSectionSize(40)
        self.tableWidget_2.verticalHeader().setDefaultSectionSize(25)
        self.label_2 = QLabel(Form)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 110, 31, 16))
        self.label_5 = QLabel(Form)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setGeometry(QRect(10, 350, 161, 21))
        self.lineEdit_3 = QLineEdit(Form)
        self.lineEdit_3.setObjectName(u"lineEdit_3")
        self.lineEdit_3.setGeometry(QRect(140, 350, 91, 24))
        self.label_6 = QLabel(Form)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setGeometry(QRect(240, 290, 49, 21))
        self.label_7 = QLabel(Form)
        self.label_7.setObjectName(u"label_7")
        self.label_7.setGeometry(QRect(240, 350, 71, 21))

        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"input interatomic potential parameters...", None))
        self.label_3.setText(QCoreApplication.translate("Form", u"cutoff:", None))
        self.label_4.setText(QCoreApplication.translate("Form", u"# of nearest neighbors:", None))
        self.lineEdit.setText(QCoreApplication.translate("Form", u"5", None))
        self.lineEdit_2.setText(QCoreApplication.translate("Form", u"3", None))
        self.OKbutton.setText(QCoreApplication.translate("Form", u"Close", None))
        self.pushButton.setText(QCoreApplication.translate("Form", u"Apply", None))
        self.label.setText(QCoreApplication.translate("Form", u"cell:", None))
        ___qtablewidgetitem = self.tableWidget.horizontalHeaderItem(0)
        ___qtablewidgetitem.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem1 = self.tableWidget.horizontalHeaderItem(1)
        ___qtablewidgetitem1.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem2 = self.tableWidget.horizontalHeaderItem(2)
        ___qtablewidgetitem2.setText(QCoreApplication.translate("Form", u"New Column", None));
        ___qtablewidgetitem3 = self.tableWidget.verticalHeaderItem(0)
        ___qtablewidgetitem3.setText(QCoreApplication.translate("Form", u"x", None));
        ___qtablewidgetitem4 = self.tableWidget.verticalHeaderItem(1)
        ___qtablewidgetitem4.setText(QCoreApplication.translate("Form", u"y", None));
        ___qtablewidgetitem5 = self.tableWidget.verticalHeaderItem(2)
        ___qtablewidgetitem5.setText(QCoreApplication.translate("Form", u"z", None));

        __sortingEnabled = self.tableWidget.isSortingEnabled()
        self.tableWidget.setSortingEnabled(False)
        ___qtablewidgetitem6 = self.tableWidget.item(0, 0)
        ___qtablewidgetitem6.setText(QCoreApplication.translate("Form", u"4.04", None));
        ___qtablewidgetitem7 = self.tableWidget.item(0, 1)
        ___qtablewidgetitem7.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem8 = self.tableWidget.item(0, 2)
        ___qtablewidgetitem8.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem9 = self.tableWidget.item(1, 0)
        ___qtablewidgetitem9.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem10 = self.tableWidget.item(1, 1)
        ___qtablewidgetitem10.setText(QCoreApplication.translate("Form", u"4.04", None));
        ___qtablewidgetitem11 = self.tableWidget.item(1, 2)
        ___qtablewidgetitem11.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem12 = self.tableWidget.item(2, 0)
        ___qtablewidgetitem12.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem13 = self.tableWidget.item(2, 1)
        ___qtablewidgetitem13.setText(QCoreApplication.translate("Form", u"0.0", None));
        ___qtablewidgetitem14 = self.tableWidget.item(2, 2)
        ___qtablewidgetitem14.setText(QCoreApplication.translate("Form", u"4.04", None));
        self.tableWidget.setSortingEnabled(__sortingEnabled)

        ___qtablewidgetitem15 = self.tableWidget_2.horizontalHeaderItem(0)
        ___qtablewidgetitem15.setText(QCoreApplication.translate("Form", u"x", None));
        ___qtablewidgetitem16 = self.tableWidget_2.horizontalHeaderItem(1)
        ___qtablewidgetitem16.setText(QCoreApplication.translate("Form", u"y", None));
        ___qtablewidgetitem17 = self.tableWidget_2.horizontalHeaderItem(2)
        ___qtablewidgetitem17.setText(QCoreApplication.translate("Form", u"z", None));
        ___qtablewidgetitem18 = self.tableWidget_2.horizontalHeaderItem(3)
        ___qtablewidgetitem18.setText(QCoreApplication.translate("Form", u"type", None));
        ___qtablewidgetitem19 = self.tableWidget_2.verticalHeaderItem(0)
        ___qtablewidgetitem19.setText(QCoreApplication.translate("Form", u"atom 1", None));
        ___qtablewidgetitem20 = self.tableWidget_2.verticalHeaderItem(1)
        ___qtablewidgetitem20.setText(QCoreApplication.translate("Form", u"atom 2", None));
        ___qtablewidgetitem21 = self.tableWidget_2.verticalHeaderItem(2)
        ___qtablewidgetitem21.setText(QCoreApplication.translate("Form", u"atom 3", None));
        ___qtablewidgetitem22 = self.tableWidget_2.verticalHeaderItem(3)
        ___qtablewidgetitem22.setText(QCoreApplication.translate("Form", u"atom 4", None));
        ___qtablewidgetitem23 = self.tableWidget_2.verticalHeaderItem(4)
        ___qtablewidgetitem23.setText(QCoreApplication.translate("Form", u"atom 5", None));
        ___qtablewidgetitem24 = self.tableWidget_2.verticalHeaderItem(5)
        ___qtablewidgetitem24.setText(QCoreApplication.translate("Form", u"atom 6", None));
        ___qtablewidgetitem25 = self.tableWidget_2.verticalHeaderItem(6)
        ___qtablewidgetitem25.setText(QCoreApplication.translate("Form", u"atom 7", None));
        ___qtablewidgetitem26 = self.tableWidget_2.verticalHeaderItem(7)
        ___qtablewidgetitem26.setText(QCoreApplication.translate("Form", u"atom 8", None));
        ___qtablewidgetitem27 = self.tableWidget_2.verticalHeaderItem(8)
        ___qtablewidgetitem27.setText(QCoreApplication.translate("Form", u"atom 9", None));
        ___qtablewidgetitem28 = self.tableWidget_2.verticalHeaderItem(9)
        ___qtablewidgetitem28.setText(QCoreApplication.translate("Form", u"atom 10", None));

        __sortingEnabled1 = self.tableWidget_2.isSortingEnabled()
        self.tableWidget_2.setSortingEnabled(False)
        ___qtablewidgetitem29 = self.tableWidget_2.item(0, 0)
        ___qtablewidgetitem29.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem30 = self.tableWidget_2.item(0, 1)
        ___qtablewidgetitem30.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem31 = self.tableWidget_2.item(0, 2)
        ___qtablewidgetitem31.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem32 = self.tableWidget_2.item(0, 3)
        ___qtablewidgetitem32.setText(QCoreApplication.translate("Form", u"Al", None));
        ___qtablewidgetitem33 = self.tableWidget_2.item(1, 0)
        ___qtablewidgetitem33.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem34 = self.tableWidget_2.item(1, 1)
        ___qtablewidgetitem34.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem35 = self.tableWidget_2.item(1, 2)
        ___qtablewidgetitem35.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem36 = self.tableWidget_2.item(1, 3)
        ___qtablewidgetitem36.setText(QCoreApplication.translate("Form", u"Al", None));
        ___qtablewidgetitem37 = self.tableWidget_2.item(2, 0)
        ___qtablewidgetitem37.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem38 = self.tableWidget_2.item(2, 1)
        ___qtablewidgetitem38.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem39 = self.tableWidget_2.item(2, 2)
        ___qtablewidgetitem39.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem40 = self.tableWidget_2.item(2, 3)
        ___qtablewidgetitem40.setText(QCoreApplication.translate("Form", u"Al", None));
        ___qtablewidgetitem41 = self.tableWidget_2.item(3, 0)
        ___qtablewidgetitem41.setText(QCoreApplication.translate("Form", u"0", None));
        ___qtablewidgetitem42 = self.tableWidget_2.item(3, 1)
        ___qtablewidgetitem42.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem43 = self.tableWidget_2.item(3, 2)
        ___qtablewidgetitem43.setText(QCoreApplication.translate("Form", u"0.5", None));
        ___qtablewidgetitem44 = self.tableWidget_2.item(3, 3)
        ___qtablewidgetitem44.setText(QCoreApplication.translate("Form", u"Al", None));
        self.tableWidget_2.setSortingEnabled(__sortingEnabled1)

        self.label_2.setText(QCoreApplication.translate("Form", u"basis:", None))
        self.label_5.setText(QCoreApplication.translate("Form", u"parameters initial guess:", None))
        self.lineEdit_3.setText("")
        self.label_6.setText(QCoreApplication.translate("Form", u"[A]", None))
        self.label_7.setText(QCoreApplication.translate("Form", u"(D, alpha, r0)", None))
    # retranslateUi




import numpy as np
import mpmath as mp
def lncomplex(z: complex) -> complex|float:
    """
    Complex natural logarithm.
    :param z: a complex number.
    :type z: complex
    :return: complex log of z
    :rtype: complex
    """
    x=np.real(z)
    _y=np.imag(z)
    _r=np.abs(z)
    return complex(np.log(_r),np.arctan2(_y, x))

def D_3(x: float|np.ndarray) -> float|np.ndarray:
    """
    Debye function with n=3.
    :param x:
    :type x: float|np.ndarray
    :return: Debye function of x with n=3.
    :rtype: float|np.ndarray
    """
    if type(x)==np.ndarray:
        return np.array([D_3(xi) for xi in x])
    if np.isnan(x):
        return 0
    if x >=499.99999999999966:
        return 1.5585456848144562e-07
    elif x>=1.579779e+01:
        return np.pi**4/(5*x**3)
    else:
        ex=np.exp(x)
        d1, d2, d3, d4, d5, d6 = -np.pi**4/(5*x**3), -3*x*(1/4), np.real(+3*lncomplex(-ex+1)),np.real(+9*mp.fp.polylog(2,ex)/x), np.real(-18*mp.fp.polylog(3,ex)/x**2), np.real(+18*mp.fp.polylog(4,ex)/x**3)
        return d1+d2+d3+d4+d5+d6
def dD_3dx(x: float|np.ndarray, D3: float|np.ndarray) -> float|np.ndarray:
    """
    Debye function derivative

    :param x:
    :type x: float|np.ndarray
    :param D3:
    :type D3: float|np.ndarray
    :return:
    :rtype: float|np.ndarray
    """
    if type(x)==np.ndarray:
        return np.array([dD_3dx(xi,D3i) for xi,D3i in zip(x,D3)])

    if x >= 709.782712893384:
        return -2.3027630998995085e-10
    return np.real(3/x * (x/(np.exp(x)-1) - D_3(x)) )

def d2D_3dx2(x: float|np.ndarray, D3: float|np.ndarray, dD3dx: float|np.ndarray) -> float|np.ndarray:
    """
    Debye function derivative.

    :param x:
    :type x: float|np.ndarray
    :param D3:
    :type D3: float|np.ndarray
    :param dD3dx:
    :type dD3dx: float|np.ndarray
    :return:
    :rtype: float|np.ndarray
    """
    if type(x)==np.ndarray:
        return np.array([d2D_3dx2(xi,D3i,dD3dxi) for xi,D3i,dD3dxi in zip(x,D3, dD3dx)])
    if x >=354.89135644669:
        exp_exp2=7.458340731215135e-155
    else:
        exp_exp2=(np.exp(x)/(np.exp(x)-1.)**2.)
    return np.real(-3.*exp_exp2+-3.*dD3dx/x+3.*D3/x**2)

def d3D_3dx3(x: float|np.ndarray, _D3: float|np.ndarray, _dD3dx: float|np.ndarray, _d2D3dx2: float|np.ndarray) -> float|np.ndarray:
    """
    Debye function derivative.

    :param x:
    :type x: float|np.ndarray
    :param _D3:
    :type _D3: float|np.ndarray
    :param _dD3dx:
    :type _dD3dx: float|np.ndarray
    :param _d2D3dx2:
    :type _d2D3dx2: float|np.ndarray
    :return:
    :rtype: float|np.ndarray
    """
    if type(x)==np.ndarray:
        return np.array([d3D_3dx3(xi,D3i,dD3dxi,d2D3dx2i) for xi,D3i,dD3dxi,d2D3dx2i in zip(x,_D3, _dD3dx, _d2D3dx2)])
    if x>=709.782712893384:
        exp_x = 1.7976931348622732e+308
    else:
        exp_x=np.exp(x)
    if x>=354.89135644669:
        exp_exp2=7.458340731215135e-155
    else:
        exp_exp2 = exp_x/(exp_x-1.)**2.
    return - 3*exp_exp2 + 6.*exp_exp2*exp_x/(exp_x-1.) - 3.*_d2D3dx2/x+6*_dD3dx/x**2 - 6.*_D3/x**3



import numpy as np
# from scipy.optimize import fmin
from scipy import optimize

from debyetools.anharmonicity import Anharmonicity, intAnharmonicity
from debyetools.electronic import Electronic
from debyetools.defects import Defects
from debyetools.vibrational import Vibrational
# import debyetools.potentials as pots
from debyetools.debfunct import D_3#, dD_3dx, d2D_3dx2, d3D_3dx3

from typing import Tuple

hbar = 0.1054571800e-33
NAv = 0.6022140857e24
kB = 0.138064852e-22


class nDeb:
    """
    Instantiate an object that contains all the parameters for the evaluation of
    the thermodynamic properties of a certain element or compound. Also contains
    the method that implements an original Debye formalism for the calculation of
    the thermodynamic properties.

    :param float nu: Poisson's ratio.
    :param float m: mass in Kg/mol-at
    :param np.ndarray p_intanh: Intrinsic anharmonicity parameters: a0, m0, V0.
    :param object EOS: Equation of state instance.
    :param np.ndarray p_electronic: Electronic contribution parameters.
    :param np.ndarray p_defects: Mono-vacancies defects contribution parameters: Evac00,Svac00,Tm,a,P2,V0.
    :param np.ndarray p_anh: Excess contribution parameters.
    :param str mode: Type of approximation of the Debye temperature (see vibrational contribution).
    """

    def __init__(self, nu: float, m: float, p_intanh: np.ndarray, EOS: object, p_electronic: np.ndarray, p_defects: np.ndarray, p_anh: np.ndarray, *args: object, units: str = 'J/mol',
                 mode: str = 'jjsl'):

        a0, m0 = p_intanh
        q0, q1, q2, q3 = p_electronic
        Evac00, Svac00, Tm, a = p_defects
        s0, s1, s2 = p_anh

        self.nu, self.r, self.m = nu, 1, m
        self.mode = mode

        self.kv = (2. / 3. * ((2. + 2. * nu) / (3. - 6. * nu)) ** (3. / 2.) + 1. / 3. * (
                    (1. + nu) / (3. - 3. * nu)) ** (3. / 2.)) ** (-1. / 3.)

        self.anh = Anharmonicity(s0, s1, s2)
        self.intanh = intAnharmonicity(a0, m0, EOS.V0)
        self.el = Electronic(q0, q1, q2, q3)
        self.deff = Defects(Evac00, Svac00, Tm, a, EOS.V0 * EOS.d2E0dV2_T(EOS.V0), EOS.V0)

        self.EOS = EOS  # getattr(pots,EOS_name)(*args,units=units, parameters = p_EOS)
        # self.EOS.pEOS = p_EOS
        self.vib = Vibrational(nu, self.EOS, m, self.intanh, mode)

        r = 1
        self.xDcte = hbar * 6 ** (1 / 3.) * (np.pi ** 2 * NAv * r) ** (1 / 3.)

    def f2min(self, T: float, V: float, P: float) -> float:
        """
        free energy for minimization.

        :param float T: Temperature.
        :param float V: Volume.
        :param float P: Pressure.

        :return: Free energy.
        :rtype: float
        """
        # self.vib.set_int_anh_4minF(T, V)
        # self.vib.set_theta_4minF(T,V)
        self.vib.set_int_anh(T, V)
        self.vib.set_theta(T, V)

        E0 = self.EOS.E0(V)
        Fvib = self.vib.F(T,V)
        Fel = self.el.F(T,V)
        Fdef = self.deff.F(T,V)
        Fa = self.anh.F(T,V)
        F = E0 + Fvib + Fel + Fdef + Fa
        return F+P*V#(dFdV_T + P)**2

    def min_G(self, T: np.ndarray, initial_V: float, P: float) -> Tuple[np.ndarray,np.ndarray]:
        """
        Procedure for the calculation of the volume as function of temperature.

        :param list_of_floats T: Temperature.
        :param float initial_V: initial guess.
        :param float P: Pressure.
        :return: Temperature and Volume
        :rtype: Tuple[np.ndarray,np.ndarray]

        """
        V0i = initial_V
        V = []
        for Ti in T[0:1]:
            f2min = lambda Vi: self.f2min(Ti, Vi, P)
            V0i = optimize.fmin(f2min, x0=V0i, disp=False)[0]
            # V0i = fmin(f2min, x0=V0i, disp=False)[0]
            V.append(V0i)
        if self.mode == '':
            pass
        else:
            self.vib.V0_DM = V[0]
        V = []
        for Ti in T:
            f2min = lambda Vi: self.f2min(Ti, Vi, P)
            # f2min = lambda Vi: 1e3*(self.dGdV_T(Ti,Vi,P=P))**2
            V0i = optimize.fmin(f2min, x0=V0i, disp=False)[0]
            V.append(V0i)


        newV = np.array(V)  # V[0]*np.exp(self.integrl())
        del V

        ixs = np.where(newV <= 1.5 * newV[0])
        # Tmax = T[-1]
        T, V = T[ixs], newV[ixs]

        return T, V

    def eval_props(self, T: np.ndarray, V: np.ndarray, P = None) -> dict:
        """
        Evaluates the thermodynamic properties of a given compound/element at (T,V).

        :param np.ndarray T: The temperature in Kelvin.
        :param np.ndarray V: The volume in "units".
        :return: A dictionary with the following keys: 'T': temperature, 'V': volume, 'tD': Debye temperature, 'g': Gruneisen parameter, 'Kt': isothermal bulk modulus, 'Ktp': pressure derivative of the isothermal bulk modulus, 'Ktpp': second order pressure derivative of the isothermal bulk modulus, 'Cv': constant-volume heat capacity, 'a': thermal expansion, 'Cp': constant-pressure heat capacity, 'Ks': adiabatic bulk modulus , 'Ksp': pressure derivative of the adiabatic bulk modulus, 'G': Gibbs free energy, 'E': total internal energy, 'S': entropy, 'E0': 'cold' internal energy defined by the EOS, 'Fvib': vibrational free energy, 'Evib': vibrational internal energy, 'Svib': vibrational entropy, 'Cvvib': vibrational heat capacity, 'Pcold': 'cold' pressure, 'dPdT_V': (dP/dT)_V, 'G^2': Ktp**2-2*Kt*Ktpp, 'dSdP_T': (dS/dP)_T, 'dKtdT_P': (dKt/dT)_P, 'dadP_T': (da/dP)_T, 'dCpdP_T': (dCp/dP)_T, 'ddSdT_PdP_T': (d2S/dTdP).
        :rtype: dict
        """
        del P
        # nu, r, m = self.nu, self.r, self.m

        # kv = self.kv

        self.vib.set_int_anh(T, V)
        self.vib.set_theta(T, V)

        E0 = self.EOS.E0(V)

        dE0dV_T   = self.EOS.dE0dV_T(V)
        d2E0dV2_T = self.EOS.d2E0dV2_T(V)
        d3E0dV3_T = self.EOS.d3E0dV3_T(V)
        d4E0dV4_T = self.EOS.d4E0dV4_T(V)
        dE0dT_V = 0
        d2E0dT2_V = 0
        d2E0dVdT = 0
        d3E0dV2dT = 0
        d3E0dVdT2 = 0

        Fvib = self.vib.F(T, V)
        Svib = -self.vib.dFdT_V(T, V)
        Evib = Fvib + T*Fvib

        dFvibdV_T = self.vib.dFdV_T(T,V)
        dFvibdT_V = self.vib.dFdT_V(T,V)
        d2FvibdT2_V = self.vib.d2FdT2_V(T,V)
        d2FvibdV2_T = self.vib.d2FdV2_T(T,V)
        d3FvibdV3_T = self.vib.d3FdV3_T(T,V)
        d4FvibdV4_T = self.vib.d4FdV4_T(T,V)
        d2FvibdVdT = self.vib.d2FdVdT(T,V)
        d3FvibdV2dT = self.vib.d3FdV2dT(T,V)
        d3FvibdVdT2 = self.vib.d3FdVdT2(T,V)

        # Eel = self.el.E(T, V)
        # Sel = self.el.S(T, V)
        Fel = self.el.F(T, V)
        dFeldV_T = self.el.dFdV_T(T, V)
        dFeldT_V = self.el.dFdT_V(T, V)
        d2FeldT2_V = self.el.d2FdT2_V(T, V)
        d2FeldV2_T = self.el.d2FdV2_T(T, V)
        d3FeldV3_T = self.el.d3FdV3_T(T, V)
        d4FeldV4_T = self.el.d4FdV4_T(T, V)
        d2FeldVdT = self.el.d2FdVdT(T, V)
        d3FeldV2dT = self.el.d3FdV2dT(T, V)
        d3FeldVdT2 = self.el.d3FdVdT2(T, V)

        # Edef = self.deff.E(T, V)
        # Sdef = self.deff.S(T, V)
        Fdef = self.deff.F(T, V)
        dFdefdV_T = self.deff.dFdV_T(T, V)
        dFdefdT_V = self.deff.dFdT_V(T, V)
        d2FdefdT2_V = self.deff.d2FdT2_V(T, V)
        d2FdefdV2_T = self.deff.d2FdV2_T(T, V)
        d3FdefdV3_T = self.deff.d3FdV3_T(T, V)
        d4FdefdV4_T = self.deff.d4FdV4_T(T, V)
        d2FdefdVdT = self.deff.d2FdVdT(T, V)
        d3FdefdV2dT = self.deff.d3FdV2dT(T, V)
        d3FdefdVdT2 = self.deff.d3FdVdT2(T, V)

        # Ea = self.anh.E(T, V)
        # Sa = self.anh.S(T, V)
        Fa = self.anh.F(T, V)
        dFadV_T = self.anh.dFdV_T(T, V)
        d2FadT2_V = self.anh.d2FdT2_V(T, V)
        dFadT_V = self.anh.dFdT_V(T, V)
        d2FadV2_T = self.anh.d2FdV2_T(T, V)
        d3FadV3_T = self.anh.d3FdV3_T(T, V)
        d4FadV4_T = self.anh.d4FdV4_T(T, V)
        d2FadVdT = self.anh.d2FdVdT(T, V)
        d3FadV2dT = self.anh.d3FdV2dT(T, V)
        d3FadVdT2 = self.anh.d3FdVdT2(T, V)

        dFdV_T = dE0dV_T + dFvibdV_T + dFeldV_T + dFdefdV_T + dFadV_T
        d2FdV2_T = d2E0dV2_T + d2FvibdV2_T + d2FeldV2_T + d2FdefdV2_T + d2FadV2_T
        d3FdV3_T = d3E0dV3_T + d3FvibdV3_T + d3FeldV3_T + d3FdefdV3_T + d3FadV3_T
        d4FdV4_T = d4E0dV4_T + d4FvibdV4_T + d4FeldV4_T + d4FdefdV4_T + d4FadV4_T
        d2FdT2_V = d2E0dT2_V + d2FvibdT2_V + d2FeldT2_V + d2FdefdT2_V + d2FadT2_V
        d2FdVdT = d2E0dVdT + d2FvibdVdT + d2FeldVdT + d2FdefdVdT + d2FadVdT
        d3FdV2dT = d3E0dV2dT + d3FvibdV2dT + d3FeldV2dT + d3FdefdV2dT + d3FadV2dT
        d3FdVdT2 = d3E0dVdT2 + d3FvibdVdT2 + d3FeldVdT2 + d3FdefdVdT2 + d3FadVdT2

        dFdT_V = dE0dT_V + dFvibdT_V + dFeldT_V + dFdefdT_V + dFadT_V

        tD = self.vib.tD
        g = -V / (tD) * self.vib.dtDdV_T
        dPdV_T = - d2FdV2_T
        d2PdV2_T = - d3FdV3_T
        d3PdV3_T = - d4FdV4_T
        Kt = - V * dPdV_T
        dKtdV_T = - dPdV_T - V * d2PdV2_T
        dKtdP_T = dKtdV_T / dPdV_T
        d2KtdV2_T = -2 * d2PdV2_T - V * d3PdV3_T
        d2KtdP2_T = (d2KtdV2_T / dPdV_T - dKtdV_T * d2PdV2_T / dPdV_T ** 2) / dPdV_T
        Ktp = dKtdP_T
        Ktpp = d2KtdP2_T
        Cv = -T * d2FdT2_V
        dPdT_V = - d2FdVdT
        dVdT_P = - dPdT_V / dPdV_T
        a = 1 / V * dVdT_P

        Cp = -T * (d2FdT2_V - (d2FdVdT) ** 2 / d2FdV2_T)

        Ks = Kt * Cp / Cv
        dCpdV_T = T * (
                    2 * d2FdVdT * d2FdV2_T * d3FdV2dT - d2FdVdT ** 2 * d3FdV3_T - d3FdVdT2 * d2FdV2_T ** 2) / d2FdV2_T ** 2
        dCvdV_T = -T * d3FdVdT2
        dKsdV_T = dKtdV_T * Cp / Cv + Kt * dCpdV_T / Cv - Kt * Cp * dCvdV_T / Cv ** 2

        dKsdP_T = dKsdV_T / dPdV_T
        Ksp = dKsdP_T

        S = -dFdT_V

        F = E0 + Fvib + Fel + Fdef + Fa
        E = F + T*S
        G = F - dFdV_T*V

        Cvvib = -T * d2FvibdT2_V
        dE0dV_T = self.EOS.dE0dV_T(V)
        Pcold = -dE0dV_T

        dSdV_T = - d2FdVdT
        dSdP_T = dSdV_T / dPdV_T

        #  _dKtdT_P = _dKtdT_V + _dKtdV_T*_dVdT_P
        d2PdVdT = - d3FdV2dT
        dKtdT_V = - V * d2PdVdT
        ddVdT_PdV_T = - d2PdVdT / dPdV_T + dPdT_V / (dPdV_T) ** 2 * d2PdV2_T
        dKtdT_P = dKtdT_V + dKtdV_T * dVdT_P
        dadV_T = -1 / V ** 2 * dVdT_P + 1 / V * ddVdT_PdV_T
        dadP_T = dadV_T / dPdV_T

        dCpdP_T = dCpdV_T / dPdV_T

        # dSdV_T = dSvibdV_T + dSdefdV_T + dSeldV_T + dSadV_T
        d2SdTdV = - d3FdVdT2  # d2SvibdVdT + d2SeldVdT + d2SdefdVdT + d2SadVdT
        d2SdV2_T = - d3FdV2dT  # _d2SvibdV2_T + d2SeldV2_T + d2SdefdV2_T + d2SadV2_T
        ddSdT_PdP_T = (d2SdTdV + d2SdV2_T * dVdT_P + dSdV_T * ddVdT_PdV_T) / dPdV_T
        return {'T': T, 'V': V, 'tD': tD, 'g': g, 'Kt': Kt, 'Ktp': Ktp, 'Ktpp': Ktpp,
                'Cv': Cv, 'a': a, 'Cp': Cp, 'Ks': Ks, 'Ksp': Ksp,
                'G': G, 'E': E, 'S': S, 'E0': E0, 'Fvib': Fvib, 'Evib': Evib, 'Svib': Svib,
                'Cvvib': Cvvib, 'Pcold': Pcold, 'dPdT_V': dPdT_V, 'G^2': Ktp ** 2 - 2 * Kt * Ktpp,
                'dSdP_T': dSdP_T, 'dKtdT_P': dKtdT_P, 'dadP_T': dadP_T, 'dCpdP_T': dCpdP_T, 'ddSdT_PdP_T': ddSdT_PdP_T,
                'P': -dFdV_T,
                'dtDdV_T':self.vib.dtDdV_T,'d2tDdV2_T':self.vib.d2tDdV2_T, 'D_3':D_3(tD/T),
                'd2E0dV2_T':d2E0dV2_T, 'dPdV_T':dPdV_T,
                'dE0dV_T':dE0dV_T, 'd3E0dV3_T':d3E0dV3_T, 'Fa':Fa, 'Fdef':Fdef, 'Fel': Fel
                }



import numpy as np

def cell(w):
    cell = np.array([[0.,0.,0.],[0.,0.,0.],[0.,0.,.0]])
    for i in range(3):
        for j in range(3):
            cell[i,j] = float(w.cellparamsTable.item(i,j).text())
    return cell

def basis(w):
    rows = w.basisTable.rowCount()
    columns = 3
    basis = np.zeros((rows, columns))
    iend = []
    for i in range(rows):
        for j in range(columns):
            try:
                basis[i,j] = float(w.basisTable.item(i,j).text())
            except:
                iend.append(i)
                break
#    print(iend, basis, basis[:iend[0],:])
    return basis[:iend[0],:]

def formula(w):
    rows = w.basisTable.rowCount()
    columns = 3

    types_list = []
    for i in range(rows):
        try:
            types_list.append(w.basisTable.item(i,columns).text())
        except:
            continue

    return types_list

def mass(w):
    mass = 0
    for el in w.molecule.types:
        mass += w.mass_dict[el]
    mass = mass/len(w.molecule.types)

    return mass

def morse_params_molecule(w):
    header = ['D', 'alpha', 'r0']
    morse_params_this = {cti:{hi:0 for hi in header} for cti in w.molecule.combs_types}
    for i in range(len(header)):
        for j in range(len(w.molecule.combs_types)):
            try:
                value = float(w.morse_params_table.item(i, j).text())
            except:
                value = ''
            morse_params_this[w.molecule.combs_types[j]][header[i]] = value

    parameters_as_list = []
    for ci in w.molecule.combs_types:
        for hi in header:
            parameters_as_list.append(morse_params_this[ci][hi])
    return morse_params_this, parameters_as_list

def C(w):
    C = np.zeros((6,6))
    for i in range(6):
        for  j in range(6):
            try:
                C[i,j] = float(w.elasticTable.item(i,j).text())
            except:
                C[i,j] = 0
    return C

def T(w):

    T_initial, T_final, Tstep = (float(sti) for sti in w.ui.lineEdit.text().split())
    T = np.arange(T_initial, T_final+Tstep, Tstep)
    T = np.r_[T, [298.15]]
    T.sort()

    return T

def P(w):
    return float(w.ui.lineEdit_2.text().replace(' ', ''))*1e9

def FS_T(w):
    return float(w.ui.lineEdit_3.text()), float(w.ui.lineEdit_4.text())




import numpy as np
from debyetools.debfunct import D_3, dD_3dx, d2D_3dx2, d3D_3dx3

np.seterr(divide='ignore',invalid='ignore')
hbar = 0.1054571800e-33
NAv = 0.6022140857e24
kB = 0.138064852e-22
r = 1


class Vibrational:
    """
    Instantiate the vibrational contribution to the free energy and its derivatives for the calculation of the
    thermodynamic properties.

    :param nu: Poisson's ratio.
    :type nu: float
    :param EOS_obj: Equation of state object.
    :type EOS_obj: potential_instance
    :param float m: Mass in Kg/mol-at.
    :param intAnharmonicity_instance intanh: Intrinsic anharmonicity object.
    """

    def __init__(self, nu: float, EOS_obj: object, m: float, intanh: np.ndarray, mode: str):
        self.d4tDdV4_T = None
        self.d3tDdVdT2 = None
        self.d3tDdV2dT = None
        self.d3tDdV3_T = None
        self.d2tDdVdT = None
        self.d2tDdT2_V = None
        self.d2tDdV2_T = None
        self.dtDdT_V = None
        self.d4AnhdV4_T = None
        self.d3AnhdV3_T = None
        self.d3AnhdVdT2 = None
        self.d3AnhdV2dT = None
        self.d2AnhdT2_V = None
        self.d2AnhdV2_T = None
        self.d2AnhdVdT = None
        self.dAnhdV_T = None
        self.dAnhdT_V = None
        self.Anh = None
        self.EOS = EOS_obj
        self.kv = (2. / 3. * ((2. + 2. * nu) / (3. - 6. * nu)) ** (3. / 2.) + 1. / 3. * (
                    (1. + nu) / (3. - 3. * nu)) ** (3. / 2.)) ** (-1. / 3.)
        self.m = m
        self.intanh = intanh
        self.DM = None
        self.tD = None
        self.dtDdV_T = None

        self.xDcte = hbar * 6 ** (1 / 3.) * (np.pi ** 2 * NAv * r) ** (1 / 3.)

        self.mode = mode
        if mode == 'DM':
            self.V0_DM = EOS_obj.V0
            self.b_DM = 0.5
            self.a_DM = -0.5
            self.lam = -1
        elif mode == 'Sl':
            self.V0_DM = EOS_obj.V0
            self.b_DM = 0.5
            self.a_DM = -1 / 6
            self.lam = -1
        elif mode == 'mfv':
            self.V0_DM = EOS_obj.V0
            self.b_DM = 0.5
            self.a_DM = -0.95
            self.lam = -1
        elif mode == 'VZ':
            self.V0_DM = EOS_obj.V0
            self.b_DM = 0.5
            self.a_DM = -5 / 6
            self.lam = -1
        elif mode == 'jjsl':
            self.V0_DM = 1
            self.b_DM = 0
            self.a_DM = 0
            self.lam = -1
        elif mode == 'jjdm':
            self.V0_DM = 1
            self.b_DM = 0
            self.a_DM = 0
            self.lam = 0
        elif mode == 'jjfv':
            self.V0_DM = 1
            self.b_DM = 0
            self.a_DM = 0
            self.lam = 1

        else:
            hola
            self.V0_DM = ''
            self.b_DM = ''
            self.a_DM = ''
            self.lam = -1

    def set_int_anh(self, T: float, V: float) -> None:
        """
        Calculates intrinsic anharmonicity correction to the Debye temperature and its derivatives.

        :param float T: Temperature.
        :param float V: Volume.
        """
        self.Anh = self.intanh.Anh(T, V)
        self.dAnhdT_V = self.intanh.dAnhdT_V()
        self.dAnhdV_T = self.intanh.dAnhdV_T(T, V)
        self.d2AnhdVdT = self.intanh.d2AnhdVdT(T)
        self.d2AnhdV2_T = self.intanh.d2AnhdV2_T(T, V)
        self.d2AnhdT2_V = self.intanh.d2AnhdT2_V()
        self.d3AnhdV2dT = self.intanh.d3AnhdV2dT(T, V)
        self.d3AnhdVdT2 = self.intanh.d3AnhdVdT2(T)
        self.d3AnhdV3_T = self.intanh.d3AnhdV3_T(T, V)
        self.d4AnhdV4_T = self.intanh.d4AnhdV4_T(T, V)
    def set_int_anh_4minF(self, T: float, V: float) -> None:
        """
        Calculates intrinsic anharmonicity correction to the Debye temperature and its derivatives.

        :param float T: Temperature.
        :param float V: Volume.
        """
        self.Anh = self.intanh.Anh(T, V)
        self.dAnhdT_V = 'X'#self.intanh.dAnhdT_V()
        self.dAnhdV_T = 'X'#self.intanh.dAnhdV_T(T, V)
        self.d2AnhdVdT = 'X'#self.intanh.d2AnhdVdT(T)
        self.d2AnhdV2_T = 'X'#self.intanh.d2AnhdV2_T(T, V)
        self.d2AnhdT2_V = 'X'#self.intanh.d2AnhdT2_V()
        self.d3AnhdV2dT = 'X'#self.intanh.d3AnhdV2dT(T, V)
        self.d3AnhdVdT2 = 'X'#self.intanh.d3AnhdVdT2(T)
        self.d3AnhdV3_T = 'X'#self.intanh.d3AnhdV3_T(T, V)
        self.d4AnhdV4_T = 'X'#self.intanh.d4AnhdV4_T(T, V)


    def set_theta(self, T: float, V: float) -> None:
        """
        Calculates the Debye Temperature and its derivatives.

        :param float T: Temperature.
        :param float V: Volume.
        """
        kv = self.kv
        m = self.m

        b_DM = self.b_DM
        a_DM = self.a_DM
        V0_DM = self.V0_DM
        lam = self.lam

        dE0dV_T = self.EOS.dE0dV_T(V)
        d2E0dV2_T = self.EOS.d2E0dV2_T(V)
        d2E0dV2_T_DM = self.EOS.d2E0dV2_T(V0_DM)
        d3E0dV3_T = self.EOS.d3E0dV3_T(V)
        d4E0dV4_T = self.EOS.d4E0dV4_T(V)
        d5E0dV5_T = self.EOS.d5E0dV5_T(V)
        d6E0dV6_T = self.EOS.d6E0dV6_T(V)

        B0_DM = V0_DM * d2E0dV2_T_DM

        P0 = - dE0dV_T
        dP0dV = - d2E0dV2_T
        dP0dV_DM = - d2E0dV2_T_DM
        d2P0dV2 = - d3E0dV3_T
        d3P0dV3 = - d4E0dV4_T
        d4P0dV4 = - d5E0dV5_T
        d5P0dV5 = - d6E0dV6_T

        vDPrm_DM = - dP0dV_DM/(r*m)
        vDsqrt_DM = np.sqrt(vDPrm_DM)
        vD_DM = kv*V0_DM*vDsqrt_DM
        xD = self.xDcte*(1/V)**(1/3.)/kB
        xD_DM = self.xDcte*(1/V0_DM)**(1/3.)/kB

        B2 = (-V*dP0dV-(2*lam*(1/3)+2/3)*P0)/(V*m*r)
        dB2dV = ((4*lam**2+14*lam+10)*P0-9*V*(-2*m*r*B2*(lam+1)*(1/3)+V*d2P0dV2))/(9*V**2*m*r)
        d2B2dV2 = ((-8*lam**3-60*lam**2-132*lam-80)*P0-27*V*(4*m*r*B2*(lam+4)*(lam+1)*(1/9)+V*(-2*m*r*(lam+1)*dB2dV*(1/3)+V*d3P0dV3)))/(27*V**3*m*r)
        d3B2dV3 = ((16*lam**4+208*lam**3+924*lam**2+1612*lam+880)*P0-81*V*(-8*r*(lam+4)*(lam+1)*(lam+11/2)*m*B2*(1/27)+V*(4*r*(lam+1)*(lam+11/2)*m*dB2dV*(1/9)+(-2*m*r*d2B2dV2*(lam+1)*(1/3)+V*d4P0dV4)*V)))/(81*V**4*m*r)
        d4B2dV4 = ((32*lam**5+256*lam**4-232*lam**3-6016*lam**2-14360*lam-8800)*P0-243*V*(-16*r*(lam-5)*(lam+4)*(lam+1)*(lam+11/2)*m*B2*(1/81)+V*(8*r*(lam-5)*(lam+1)*(lam+11/2)*m*dB2dV*(1/27)+(-4*m*r*(lam+1)*(lam-5)*d2B2dV2*(1/9)+V*(2*m*r*(lam+1)*d3B2dV3*(1/3)+V*d5P0dV5))*V)))/(243*V**5*m*r)

        vD = V*kv*np.sqrt(B2)
        dvDdV = (V**3*kv**2*(dB2dV)+2*vD**2)/(2*V*vD)
        d2vDdV2 = (V**4*kv**2*(d2B2dV2)-2*dvDdV**2*V**2+8*dvDdV*V*vD-6*vD**2)/(2*V**2*vD)
        d3vDdV3 = (V**5*kv**2*(d3B2dV3)-(6*(2*vD+V*(V*d2vDdV2-2*dvDdV)))*(dvDdV*V-2*vD))/(2*vD*V**3)
        d4vDdV4 = ((d4B2dV4)*kv**2*V**6-120*vD**2+(16*V**3*d3vDdV3-72*d2vDdV2*V**2+192*dvDdV*V)*vD-72*dvDdV**2*V**2-8*V**3*(V*d3vDdV3-6*d2vDdV2)*dvDdV-6*d2vDdV2**2*V**4)/(2*V**4*vD)
        dxDdV = -self.xDcte/(3*kB*V**(4/3.))
        d2xDdV2 = 4*self.xDcte/(9*kB*V**(7/3.))
        d3xDdV3 = -28*self.xDcte/(27*V**(10/3)*kB)
        d4xDdV4 = 280*self.xDcte/(81*V**(13/3)*kB)

        DM = (V*d2E0dV2_T/B0_DM)**b_DM/(V/V0_DM)**a_DM
        self.DM = DM
        dDMdV = (V*d2E0dV2_T/B0_DM)**b_DM*b_DM*(d2E0dV2_T/B0_DM+V*(d3E0dV3_T)/B0_DM)*B0_DM/(V*d2E0dV2_T*(V/V0_DM)**a_DM)-(V*d2E0dV2_T/B0_DM)**b_DM*a_DM/((V/V0_DM)**a_DM*V)
        d2DMdV2 = V0_DM**a_DM*B0_DM**(-b_DM)*(-2*(d3E0dV3_T)*b_DM*d2E0dV2_T**(b_DM-1)*(-b_DM+a_DM)*V**(b_DM-1-a_DM)+d2E0dV2_T**b_DM*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-2-a_DM)+V**(b_DM-a_DM)*(d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*b_DM)
        d3DMdV3 = -V0_DM**a_DM*B0_DM**(-b_DM)*(d2E0dV2_T**b_DM*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-a_DM-3)-3*b_DM*(-(d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*(-b_DM+a_DM)*V**(b_DM-1-a_DM)+(d3E0dV3_T)*d2E0dV2_T**(b_DM-1)*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-2-a_DM)+(1/3)*V**(b_DM-a_DM)*(d2E0dV2_T**(b_DM-1)*(d5E0dV5_T)+(d3E0dV3_T)*(3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-3)*(b_DM-2))*(b_DM-1))))
        d4DMdV4 = (d2E0dV2_T**b_DM*(-b_DM+a_DM)*(-b_DM+a_DM+3)*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*V**(b_DM-4-a_DM)+6*b_DM*((d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*(-b_DM+a_DM)*(-b_DM+1+a_DM)*V**(b_DM-2-a_DM)-(1/3)*(2*(d2E0dV2_T**(b_DM-1)*(d5E0dV5_T)+(d3E0dV3_T)*(3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-3)*(b_DM-2))*(b_DM-1)))*(-b_DM+a_DM)*V**(b_DM-1-a_DM)-2*(d3E0dV3_T)*d2E0dV2_T**(b_DM-1)*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*(- b_DM +a_DM)*V**(b_DM-a_DM-3)*(1/3)+(1/6)*(d2E0dV2_T**(b_DM-1)*(d6E0dV6_T)+(4*(d3E0dV3_T)*d2E0dV2_T**(b_DM-2)*(d5E0dV5_T)+3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)**2+(6*d2E0dV2_T**(b_DM-3)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-4)*(b_DM-3))*(b_DM-2)*(d3E0dV3_T)**2)*(b_DM-1))*V**(b_DM-a_DM)))*V0_DM**a_DM*B0_DM**(-b_DM)

        tD_DM = xD_DM * vD_DM
        self.tD = xD*vD*self.Anh if 'jj' in self.mode else tD_DM*self.Anh*DM
        self.dtDdV_T = dxDdV*vD*self.Anh+xD*dvDdV*self.Anh+xD*vD*self.dAnhdV_T if 'jj' in self.mode else tD_DM*self.Anh*dDMdV
        self.dtDdT_V = xD*vD*self.dAnhdT_V if 'jj' in self.mode else 0
        self.d2tDdV2_T =d2xDdV2*vD*self.Anh+2*dxDdV*dvDdV*self.Anh+2*dxDdV*vD*self.dAnhdV_T+xD*d2vDdV2*self.Anh+2*xD*dvDdV*self.dAnhdV_T+xD*vD*self.d2AnhdV2_T if 'jj' in self.mode else tD_DM*self.Anh*d2DMdV2
        self.d2tDdT2_V = xD*vD*self.d2AnhdT2_V if 'jj' in self.mode else 0
        self.d2tDdVdT = dxDdV*vD*self.dAnhdT_V+xD*dvDdV*self.dAnhdT_V+xD*vD*self.d2AnhdVdT if 'jj' in self.mode else 0
        self.d3tDdV3_T = d3xDdV3*vD*self.Anh+3*d2xDdV2*dvDdV*self.Anh+3*d2xDdV2*vD*self.dAnhdV_T+3*dxDdV*d2vDdV2*self.Anh+6*dxDdV*dvDdV*self.dAnhdV_T+3*dxDdV*vD*self.d2AnhdV2_T+xD*d3vDdV3*self.Anh+3*xD*d2vDdV2*self.dAnhdV_T+3*xD*dvDdV*self.d2AnhdV2_T+xD*vD*self.d3AnhdV3_T  if 'jj' in self.mode else tD_DM*self.Anh*d3DMdV3
        self.d3tDdV2dT =d2xDdV2*vD*self.dAnhdT_V+2*dxDdV*dvDdV*self.dAnhdT_V+2*dxDdV*vD*self.d2AnhdVdT+xD*d2vDdV2*self.dAnhdT_V+2*xD*dvDdV*self.d2AnhdVdT+xD*vD*self.d3AnhdV2dT  if 'jj' in self.mode else 0
        self.d3tDdVdT2 = dxDdV*vD*self.d2AnhdT2_V+xD*dvDdV*self.d2AnhdT2_V+xD*vD*self.d3AnhdVdT2  if 'jj' in self.mode else 0
        self.d4tDdV4_T = 6*d2xDdV2*d2vDdV2*self.Anh+12*d2xDdV2*dvDdV*self.dAnhdV_T+6*d2xDdV2*vD*self.d2AnhdV2_T+4*dxDdV*d3vDdV3*self.Anh+12*dxDdV*d2vDdV2*self.dAnhdV_T+12*dxDdV*dvDdV*self.d2AnhdV2_T+4*dxDdV*vD*self.d3AnhdV3_T+xD*d4vDdV4*self.Anh+4*xD*d3vDdV3*self.dAnhdV_T+6*xD*d2vDdV2*self.d2AnhdV2_T+4*xD*dvDdV*self.d3AnhdV3_T+xD*vD*self.d4AnhdV4_T+d4xDdV4*vD*self.Anh+4*d3xDdV3*dvDdV*self.Anh+4*d3xDdV3*vD*self.dAnhdV_T  if 'jj' in self.mode else tD_DM*self.Anh*d4DMdV4

    def set_theta_4minF(self, T: float, V: float) -> None:
        """
        Calculates the Debye Temperature and its derivatives.

        :param float T: Temperature.
        :param float V: Volume.
        """
        kv = self.kv
        m = self.m
        #
        b_DM = self.b_DM
        a_DM = self.a_DM
        V0_DM = self.V0_DM
        lam = self.lam
        #
        dE0dV_T = self.EOS.dE0dV_T(V)
        d2E0dV2_T = self.EOS.d2E0dV2_T(V)
        d2E0dV2_T_DM = self.EOS.d2E0dV2_T(V0_DM)
        # d3E0dV3_T = self.EOS.d3E0dV3_T(V)
        # d4E0dV4_T = self.EOS.d4E0dV4_T(V)
        # d5E0dV5_T = self.EOS.d5E0dV5_T(V)
        # d6E0dV6_T = self.EOS.d6E0dV6_T(V)
        #
        B0_DM = V0_DM * d2E0dV2_T_DM
        #
        P0 = - dE0dV_T
        dP0dV = - d2E0dV2_T
        dP0dV_DM = - d2E0dV2_T_DM
        # d2P0dV2 = - d3E0dV3_T
        # d3P0dV3 = - d4E0dV4_T
        # d4P0dV4 = - d5E0dV5_T
        # d5P0dV5 = - d6E0dV6_T
        #
        vDPrm_DM = - dP0dV_DM/(r*m)
        vDsqrt_DM = np.sqrt(vDPrm_DM)
        vD_DM = kv*V0_DM*vDsqrt_DM
        xD = self.xDcte*(1/V)**(1/3.)/kB
        xD_DM = self.xDcte*(1/V0_DM)**(1/3.)/kB
        #
        B2 = (-V*dP0dV-(2*lam*(1/3)+2/3)*P0)/(V*m*r)
        # dB2dV = ((4*lam**2+14*lam+10)*P0-9*V*(-2*m*r*B2*(lam+1)*(1/3)+V*d2P0dV2))/(9*V**2*m*r)
        # d2B2dV2 = ((-8*lam**3-60*lam**2-132*lam-80)*P0-27*V*(4*m*r*B2*(lam+4)*(lam+1)*(1/9)+V*(-2*m*r*(lam+1)*dB2dV*(1/3)+V*d3P0dV3)))/(27*V**3*m*r)
        # d3B2dV3 = ((16*lam**4+208*lam**3+924*lam**2+1612*lam+880)*P0-81*V*(-8*r*(lam+4)*(lam+1)*(lam+11/2)*m*B2*(1/27)+V*(4*r*(lam+1)*(lam+11/2)*m*dB2dV*(1/9)+(-2*m*r*d2B2dV2*(lam+1)*(1/3)+V*d4P0dV4)*V)))/(81*V**4*m*r)
        # d4B2dV4 = ((32*lam**5+256*lam**4-232*lam**3-6016*lam**2-14360*lam-8800)*P0-243*V*(-16*r*(lam-5)*(lam+4)*(lam+1)*(lam+11/2)*m*B2*(1/81)+V*(8*r*(lam-5)*(lam+1)*(lam+11/2)*m*dB2dV*(1/27)+(-4*m*r*(lam+1)*(lam-5)*d2B2dV2*(1/9)+V*(2*m*r*(lam+1)*d3B2dV3*(1/3)+V*d5P0dV5))*V)))/(243*V**5*m*r)
        #
        vD = V*kv*np.sqrt(B2)
        # dvDdV = (V**3*kv**2*(dB2dV)+2*vD**2)/(2*V*vD)
        # d2vDdV2 = (V**4*kv**2*(d2B2dV2)-2*dvDdV**2*V**2+8*dvDdV*V*vD-6*vD**2)/(2*V**2*vD)
        # d3vDdV3 = (V**5*kv**2*(d3B2dV3)-(6*(2*vD+V*(V*d2vDdV2-2*dvDdV)))*(dvDdV*V-2*vD))/(2*vD*V**3)
        # d4vDdV4 = ((d4B2dV4)*kv**2*V**6-120*vD**2+(16*V**3*d3vDdV3-72*d2vDdV2*V**2+192*dvDdV*V)*vD-72*dvDdV**2*V**2-8*V**3*(V*d3vDdV3-6*d2vDdV2)*dvDdV-6*d2vDdV2**2*V**4)/(2*V**4*vD)
        # dxDdV = -self.xDcte/(3*kB*V**(4/3.))
        # d2xDdV2 = 4*self.xDcte/(9*kB*V**(7/3.))
        # d3xDdV3 = -28*self.xDcte/(27*V**(10/3)*kB)
        # d4xDdV4 = 280*self.xDcte/(81*V**(13/3)*kB)
        #
        DM = (V*d2E0dV2_T/B0_DM)**b_DM/(V/V0_DM)**a_DM
        # self.DM = DM
        # dDMdV = (V*d2E0dV2_T/B0_DM)**b_DM*b_DM*(d2E0dV2_T/B0_DM+V*(d3E0dV3_T)/B0_DM)*B0_DM/(V*d2E0dV2_T*(V/V0_DM)**a_DM)-(V*d2E0dV2_T/B0_DM)**b_DM*a_DM/((V/V0_DM)**a_DM*V)
        # d2DMdV2 = V0_DM**a_DM*B0_DM**(-b_DM)*(-2*(d3E0dV3_T)*b_DM*d2E0dV2_T**(b_DM-1)*(-b_DM+a_DM)*V**(b_DM-1-a_DM)+d2E0dV2_T**b_DM*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-2-a_DM)+V**(b_DM-a_DM)*(d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*b_DM)
        # d3DMdV3 = -V0_DM**a_DM*B0_DM**(-b_DM)*(d2E0dV2_T**b_DM*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-a_DM-3)-3*b_DM*(-(d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*(-b_DM+a_DM)*V**(b_DM-1-a_DM)+(d3E0dV3_T)*d2E0dV2_T**(b_DM-1)*(-b_DM+1+a_DM)*(-b_DM+a_DM)*V**(b_DM-2-a_DM)+(1/3)*V**(b_DM-a_DM)*(d2E0dV2_T**(b_DM-1)*(d5E0dV5_T)+(d3E0dV3_T)*(3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-3)*(b_DM-2))*(b_DM-1))))
        # d4DMdV4 = (d2E0dV2_T**b_DM*(-b_DM+a_DM)*(-b_DM+a_DM+3)*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*V**(b_DM-4-a_DM)+6*b_DM*((d2E0dV2_T**(b_DM-1)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-2)*(b_DM-1))*(-b_DM+a_DM)*(-b_DM+1+a_DM)*V**(b_DM-2-a_DM)-(1/3)*(2*(d2E0dV2_T**(b_DM-1)*(d5E0dV5_T)+(d3E0dV3_T)*(3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-3)*(b_DM-2))*(b_DM-1)))*(-b_DM+a_DM)*V**(b_DM-1-a_DM)-2*(d3E0dV3_T)*d2E0dV2_T**(b_DM-1)*(-b_DM+2+a_DM)*(-b_DM+1+a_DM)*(- b_DM +a_DM)*V**(b_DM-a_DM-3)*(1/3)+(1/6)*(d2E0dV2_T**(b_DM-1)*(d6E0dV6_T)+(4*(d3E0dV3_T)*d2E0dV2_T**(b_DM-2)*(d5E0dV5_T)+3*d2E0dV2_T**(b_DM-2)*(d4E0dV4_T)**2+(6*d2E0dV2_T**(b_DM-3)*(d4E0dV4_T)+(d3E0dV3_T)**2*d2E0dV2_T**(b_DM-4)*(b_DM-3))*(b_DM-2)*(d3E0dV3_T)**2)*(b_DM-1))*V**(b_DM-a_DM)))*V0_DM**a_DM*B0_DM**(-b_DM)

        tD_DM = xD_DM * vD_DM
        self.tD = xD*vD*self.Anh if 'jj' in self.mode else tD_DM*self.Anh*DM
        # self.dtDdV_T = 'X'#dxDdV*vD*self.Anh+xD*dvDdV*self.Anh+xD*vD*self.dAnhdV_T if 'jj' in self.mode else tD_DM*self.Anh*dDMdV
        # self.dtDdT_V = 'X'#xD*vD*self.dAnhdT_V if 'jj' in self.mode else 0
        # self.d2tDdV2_T ='X'#d2xDdV2*vD*self.Anh+2*dxDdV*dvDdV*self.Anh+2*dxDdV*vD*self.dAnhdV_T+xD*d2vDdV2*self.Anh+2*xD*dvDdV*self.dAnhdV_T+xD*vD*self.d2AnhdV2_T if 'jj' in self.mode else tD_DM*self.Anh*d2DMdV2
        # self.d2tDdT2_V = 'X'#xD*vD*self.d2AnhdT2_V if 'jj' in self.mode else 0
        # self.d2tDdVdT = 'X'#dxDdV*vD*self.dAnhdT_V+xD*dvDdV*self.dAnhdT_V+xD*vD*self.d2AnhdVdT if 'jj' in self.mode else 0
        # self.d3tDdV3_T = 'X'#d3xDdV3*vD*self.Anh+3*d2xDdV2*dvDdV*self.Anh+3*d2xDdV2*vD*self.dAnhdV_T+3*dxDdV*d2vDdV2*self.Anh+6*dxDdV*dvDdV*self.dAnhdV_T+3*dxDdV*vD*self.d2AnhdV2_T+xD*d3vDdV3*self.Anh+3*xD*d2vDdV2*self.dAnhdV_T+3*xD*dvDdV*self.d2AnhdV2_T+xD*vD*self.d3AnhdV3_T  if 'jj' in self.mode else tD_DM*self.Anh*d3DMdV3
        # self.d3tDdV2dT ='X'#d2xDdV2*vD*self.dAnhdT_V+2*dxDdV*dvDdV*self.dAnhdT_V+2*dxDdV*vD*self.d2AnhdVdT+xD*d2vDdV2*self.dAnhdT_V+2*xD*dvDdV*self.d2AnhdVdT+xD*vD*self.d3AnhdV2dT  if 'jj' in self.mode else 0
        # self.d3tDdVdT2 = 'X'#dxDdV*vD*self.d2AnhdT2_V+xD*dvDdV*self.d2AnhdT2_V+xD*vD*self.d3AnhdVdT2  if 'jj' in self.mode else 0
        # self.d4tDdV4_T = 'X'#6*d2xDdV2*d2vDdV2*self.Anh+12*d2xDdV2*dvDdV*self.dAnhdV_T+6*d2xDdV2*vD*self.d2AnhdV2_T+4*dxDdV*d3vDdV3*self.Anh+12*dxDdV*d2vDdV2*self.dAnhdV_T+12*dxDdV*dvDdV*self.d2AnhdV2_T+4*dxDdV*vD*self.d3AnhdV3_T+xD*d4vDdV4*self.Anh+4*xD*d3vDdV3*self.dAnhdV_T+6*xD*d2vDdV2*self.d2AnhdV2_T+4*xD*dvDdV*self.d3AnhdV3_T+xD*vD*self.d4AnhdV4_T+d4xDdV4*vD*self.Anh+4*d3xDdV3*dvDdV*self.Anh+4*d3xDdV3*vD*self.dAnhdV_T  if 'jj' in self.mode else tD_DM*self.Anh*d4DMdV4



    def F(self, T: float, V: float) -> float:
        """
        Vibration Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: F_vib.
        :rtype:  float
        """

        x = self.tD/T
        D3 = D_3(x)
        # print(tD/T, tD, T)
        if type(V) is not np.ndarray:
            if x < 0.04:
                return 1e10
        return 3*r*NAv*kB*(self.tD*3/8+T*np.log(1-np.exp(-x))-D3*T/3)

    def dFdV_T(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: dFdV_T.
        :rtype:  float
        """

        x = self.tD/T
        D3 = D_3(x)
        dD3 = dD_3dx(x, D3)
        return 3*r*NAv*kB*(3*(self.dtDdV_T)*(1/8)+(self.dtDdV_T)*np.exp(-x)/(1-np.exp(-x))-(1/3)*dD3*(self.dtDdV_T))

    def dFdT_V(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: dFdT_V.
        :rtype:  float
        """

        x = self.tD / T
        ixs = np.where(x >= 653)
        if len(ixs[0]) > 0:
            if min(x[ixs]) >= 653:
                for i in ixs:
                    x[i] = 653
        ex = np.exp(x)
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        return 9*r*NAv*kB*(self.dtDdT_V)*(1/8) + 3*kB*r*NAv*np.log(1-np.exp(-x)) + 3*r*NAv*kB*(self.dtDdT_V)/(ex*(1-1/ex)) - 3*r*NAv*kB*self.tD/(T*ex*(1-1/ex)) - r*NAv*kB*dD3dx*(self.dtDdT_V) + r*NAv*kB*dD3dx*self.tD/T - r*NAv*kB*D3

    def d2FdT2_V(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d2FdT2_V.
        :rtype:  float
        """

        x = self.tD / T
        ixs = np.where(x >= 653)
        if len(ixs[0]) > 0:
            if min(x[ixs]) >= 653:
                for i in ixs:
                    x[i] = 653
        ex = np.exp(x)
        D3 = D_3(x)
        return 3 * r * NAv * ((ex - 1) * T * (
                    T ** 2 * self.d2tDdT2_V * self.tD - 4 * (self.dtDdT_V * T - self.tD) ** 2) * D3 + 3 * self.tD * (
                             self.d2tDdT2_V * self.tD * ex * T ** 2 - T ** 2 * self.d2tDdT2_V * self.tD + 8 * (
                              self.dtDdT_V * T - self.tD) ** 2) * (1 / 8)) * kB / (
                               self.tD ** 2 * (ex - 1) * T ** 2)

    def d2FdV2_T(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d2FdV2_T.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        return 3 * r * NAv * kB * (
                    8 * self.dtDdV_T ** 2 * self.tD * dD3dx - 8 * self.dtDdV_T ** 2 * D3 * T + 8 * self.d2tDdV2_T * D3 * self.tD * T + 3 * self.d2tDdV2_T * self.tD ** 2) / (
                           8 * self.tD ** 2)

    def d3FdV3_T(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d3FdV3_T.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        d2D3dx2 = d2D_3dx2(x, D3, dD3dx)
        return 3 * r * (T ** 2 * (
                    self.d3tDdV3_T * self.tD ** 2 - 3 * self.dtDdV_T * self.d2tDdV2_T * self.tD + 2 * self.dtDdV_T ** 3) * D3 + (
                                    3 * self.dtDdV_T * self.d2tDdV2_T * T * self.tD ** 2 - 2 * self.dtDdV_T ** 3 * T * self.tD) * dD3dx + d2D3dx2 * self.dtDdV_T ** 3 * self.tD ** 2 + 3 * self.d3tDdV3_T * self.tD ** 3 * T * (
                                    1 / 8)) * kB * NAv / (T * self.tD ** 3)

    def d4FdV4_T(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d4FdV4_T.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        d2D3dx2 = d2D_3dx2(x, D3, dD3dx)
        d3D3dx3 = d3D_3dx3(x, D3, dD3dx, d2D3dx2)

        return -12 * r * (T ** 3 * (-(1 / 4) * self.d4tDdV4_T * self.tD ** 3 + (
                    self.d3tDdV3_T * self.dtDdV_T + 3 * self.d2tDdV2_T ** 2 * (
                        1 / 4)) * self.tD ** 2 - 3 * self.dtDdV_T ** 2 * self.d2tDdV2_T * self.tD + 3 * self.dtDdV_T ** 4 * (
                                                1 / 2)) * D3 - self.tD * (T ** 2 * ((
                                                                                                self.d3tDdV3_T * self.dtDdV_T + 3 * self.d2tDdV2_T ** 2 * (
                                                                                                    1 / 4)) * self.tD ** 2 - 3 * self.dtDdV_T ** 2 * self.d2tDdV2_T * self.tD + 3 * self.dtDdV_T ** 4 * (
                                                                                                1 / 2)) * dD3dx + (
                                                                                      1 / 32) * (3 * ((
                                                                                                                  16 * self.dtDdV_T ** 2 * self.d2tDdV2_T * self.tD * T - 8 * self.dtDdV_T ** 4 * T) * d2D3dx2 + self.tD * (
                                                                                                                  8 * d3D3dx3 * self.dtDdV_T ** 4 * (
                                                                                                                      1 / 3) + self.d4tDdV4_T * self.tD * T ** 2))) * self.tD)) * kB * NAv / (
                           T ** 2 * self.tD ** 4)

    def d2FdVdT(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d2FdVdT.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        return 3 * r * (dD3dx * (self.dtDdT_V / T - self.tD / T ** 2) * T + D3 + 3 * self.dtDdT_V * (
                    1 / 8)) * kB * self.dtDdV_T * NAv / self.tD + 3 * r * (
                           D3 * T + 3 * self.tD * (1 / 8)) * kB * self.d2tDdVdT * NAv / self.tD - 3 * r * (
                           D3 * T + 3 * self.tD * (1 / 8)) * kB * self.dtDdV_T * NAv * self.dtDdT_V / self.tD ** 2

    def d3FdV2dT(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d3FdV2dT.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        d2D3dx2 = d2D_3dx2(x, D3, dD3dx)

        return -3 * r * (T ** 2 * ((-T * self.d3tDdV2dT - self.d2tDdV2_T) * self.tD ** 2 + (
                    T * self.d2tDdV2_T * self.dtDdT_V + 2 * T * self.dtDdV_T * self.d2tDdVdT + self.dtDdV_T ** 2) * self.tD - 2 * T * self.dtDdT_V * self.dtDdV_T ** 2) * D3 - self.tD * (
                                     (-self.d2tDdV2_T * self.tD ** 2 + (
                                                 T * self.d2tDdV2_T * self.dtDdT_V + 2 * T * self.dtDdV_T * self.d2tDdVdT + self.dtDdV_T ** 2) * self.tD - 2 * T * self.dtDdT_V * self.dtDdV_T ** 2) * T * dD3dx + 3 * self.tD * (
                                                 8 * self.dtDdV_T ** 2 * (T * self.dtDdT_V - self.tD) * d2D3dx2 * (
                                                     1 / 3) + self.tD * self.d3tDdV2dT * T ** 2) * (
                                                 1 / 8))) * kB * NAv / (T ** 2 * self.tD ** 3)

    def d3FdVdT2(self, T: float, V: float) -> float:
        """
        Derivative of vibrational Helmholtz free energy.

        :param float T: Temperature.
        :param float V: Volume.
        :return: d3FdVdT2.
        :rtype:  float
        """
        x = self.tD / T
        D3 = D_3(x)
        dD3dx = dD_3dx(x, D3)
        d2D3dx2 = d2D_3dx2(x, D3, dD3dx)

        return -(6 * (T ** 3 * ((-1/2 * self.d3tDdVdT2 * T - self.d2tDdVdT) * self.tD ** 2 + ((1/2 * self.d2tDdT2_V * T + self.dtDdT_V) * self.dtDdV_T + self.dtDdT_V * self.d2tDdVdT * T) * self.tD - self.dtDdT_V ** 2 * self.dtDdV_T * T) * D3 - (
                                  T ** 2 * (-self.tD ** 2 * self.d2tDdVdT + (((
                                                                                          1 / 2) * self.d2tDdT2_V * T + self.dtDdT_V) * self.dtDdV_T + self.dtDdT_V * self.d2tDdVdT * T) * self.tD - self.dtDdT_V ** 2 * self.dtDdV_T * T) * dD3dx + (
                                              1 / 16) * (3 * (
                                      8 * self.dtDdV_T * (T * self.dtDdT_V - self.tD) ** 2 * d2D3dx2 * (
                                          1 / 3) + self.tD * self.d3tDdVdT2 * T ** 3)) * self.tD) * self.tD)) * r * kB * NAv / (
                           T ** 3 * self.tD ** 3)



import PySimpleGUI as sg
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
plt.switch_backend('TkAgg')
import numpy as np
# import addcopyfighandler
#
legend_loc_list = ['best','upper right','upper left','lower left','lower right','right','center left','center right','lower center','upper center','center']
line_styles_list = ['-','--','-.',':','None']
colors_list = ['mediumpurple','purple','gray','orchid','deepskyblue', 'pink', 'darkkhaki', 'aqua','cornflowerblue','blue','green','red','C0','C1','C2','C3','C4']
markers_list=['.','o','+','x','1', '2', '3', '4','s','h','^', 'v','>', '<','*', '8', 'd', 'P', 'p', 'H','None']
#
def NewSpinner(init_val=None,width=None,key=None,tooltip=None):
    return [sg.Input(init_val, size=(width, 1), font='Any 10', justification='l', key=key,tooltip=tooltip),
    sg.Column([[sg.Button('▲', size=(1, 1), font='Any 7', border_width=0, button_color=(sg.theme_text_color(), sg.theme_background_color()), key='--B_'+key+'_UP')],
               [sg.Button('▼', size=(1, 1), font='Any 7', border_width=0, button_color=(sg.theme_text_color(), sg.theme_background_color()), key='--B_'+key+'_DN')]])]
#
def textSpinner(txt=None,init_val=None,width=None,key=None,tooltip=None):
    spinner = NewSpinner(init_val=init_val,width=width,key=key,tooltip=tooltip)
    return sg.Text(txt,key='--T_'+key), spinner[0], spinner[1]
#
def general_layout(lines,jx):
    figwidth_spin = textSpinner(txt='  w: ',init_val='6',width=3,key='ix'+str(jx)+'-'+'figwidth')
    figheight_spin = textSpinner(txt='  h: ',init_val='4',width=3,key='ix'+str(jx)+'-'+'figheight')
    layout_figure = [[sg.Text('  Figure size: ',key='--T_figure_size'),
                      figwidth_spin[0],figwidth_spin[1],figwidth_spin[2], figheight_spin[0],figheight_spin[1],figheight_spin[2]]]

    titlesize_spin = textSpinner(txt='  font size: ',init_val='10',width=3,key='ix'+str(jx)+'-'+'titlesize')
    titlexpos_spin = textSpinner(txt='  x: ',init_val='0.5',width=4,key='ix'+str(jx)+'-'+'titlexpos')
    titleypos_spin = textSpinner(txt='  y: ',init_val='0.98',width=4,key='ix'+str(jx)+'-'+'titleypos')
    layout_title = [[sg.Checkbox('use title', enable_events=True, key='ix'+str(jx)+'-'+'use_title')],
                    [sg.Text('  title: ',key='--T_title'),sg.Input(size=(14, 1), justification='right', key='ix'+str(jx)+'-'+'title'),
                     titlesize_spin[0],titlesize_spin[1],titlesize_spin[2]],
                    [sg.Text('  Position: ',key='--T_pos_title'),
                     titlexpos_spin[0],titlexpos_spin[1],titlexpos_spin[2], titleypos_spin[0],titleypos_spin[1],titleypos_spin[2]]]

    labelxsize_spin = textSpinner(txt='  font size: ',init_val='10',width=3,key='ix'+str(jx)+'-'+'labelxsize')
    labelysize_spin = textSpinner(txt='  font size: ',init_val='10',width=3,key='ix'+str(jx)+'-'+'labelysize')

    limxmin_spin = textSpinner(txt='  min: ',init_val='0',width=4,key='ix'+str(jx)+'-'+'limxmin')
    limymin_spin = textSpinner(txt='  min: ',init_val='0',width=4,key='ix'+str(jx)+'-'+'limymin')
    limxmax_spin = textSpinner(txt='  max: ',init_val='0',width=4,key='ix'+str(jx)+'-'+'limxmax')
    limymax_spin = textSpinner(txt='  max: ',init_val='0',width=4,key='ix'+str(jx)+'-'+'limymax')

    layout_axis = [[sg.Checkbox('use xlabel ', enable_events=True, key='ix'+str(jx)+'-'+'use_xlabel'),sg.Checkbox('use ylabel ', enable_events=True, key='ix'+str(jx)+'-'+'use_ylabel')],
                   [sg.Text('  xlabel: ',key='--T_x_label'),sg.Input(size=(14, 1), justification='right', key='ix'+str(jx)+'-'+'xlabel'),
                   labelxsize_spin[0],labelxsize_spin[1],labelxsize_spin[2]],
                   [sg.Text('  ylabel: ',key='--T_y_label'),sg.Input(size=(14, 1), justification='right', key='ix'+str(jx)+'-'+'ylabel'),
                   labelysize_spin[0],labelysize_spin[1],labelysize_spin[2]],
                   [sg.Checkbox('auto xlim ', enable_events=True, key='ix'+str(jx)+'-'+'auto_xlim')],
                   [sg.Text('  xlim: ',key='--T_xlim'),
                    limxmin_spin[0],limxmin_spin[1],limxmin_spin[2], limxmax_spin[0],limxmax_spin[1],limxmax_spin[2]],
                   [sg.Checkbox('auto ylim ', enable_events=True, key='ix'+str(jx)+'-'+'auto_ylim')],[
                    sg.Text('  ylim: ',key='--T_ylim'),
                    limymin_spin[0],limymin_spin[1],limymin_spin[2], limymax_spin[0],limymax_spin[1],limymax_spin[2]]]

    legendncol_spin     = textSpinner(txt='  columns: ',init_val='2',width=3,key='ix'+str(jx)+'-'+'legendncol')
    legendfontsize_spin = textSpinner(txt='  font size: ',init_val='10',width=3,key='ix'+str(jx)+'-'+'legendfontsize')
    layout_legend = [[sg.Checkbox('use legend ', enable_events=True, key='ix'+str(jx)+'-'+'use_legend'),sg.Text('  location: ',key='--T_legend_loc'),
                      sg.InputCombo(values=legend_loc_list, size=(12,1), key='ix'+str(jx)+'-'+'legend_loc')],
                     [legendncol_spin[0],legendncol_spin[1],legendncol_spin[2], legendfontsize_spin[0],legendfontsize_spin[1],legendfontsize_spin[2]]]
    layout_grid = [[sg.Checkbox('use grid ', enable_events=True, key='ix'+str(jx)+'-'+'use_grid')]]

    lmargin_spin = textSpinner(txt='  left: ',init_val='0.18',width=4,key='ix'+str(jx)+'-'+'lmargin')
    rmargin_spin = textSpinner(txt='  riht: ',init_val='0.98',width=4,key='ix'+str(jx)+'-'+'rmargin')
    bmargin_spin = textSpinner(txt='  bottom: ',init_val='0.12',width=4,key='ix'+str(jx)+'-'+'bmargin')
    tmargin_spin = textSpinner(txt='  top: ',init_val='0.95',width=4,key='ix'+str(jx)+'-'+'tmargin')
    layout_margins = [[lmargin_spin[0],lmargin_spin[1],lmargin_spin[2], rmargin_spin[0],rmargin_spin[1],rmargin_spin[2],
                       bmargin_spin[0],bmargin_spin[1],bmargin_spin[2], tmargin_spin[0],tmargin_spin[1],tmargin_spin[2]]]

    layout_left = [[sg.Frame(title='Figure',layout=layout_figure,key='--Frame_figure')],
                   [sg.Frame(title='Title',layout=layout_title,key='--Frame_title')],
                   [sg.Frame(title='Axis',layout=layout_axis,key='--Frame_axis')],
                   [sg.Frame(title='Legend',layout=layout_legend,key='--Frame_legend')],
                   [sg.Frame(title='Grid',layout=layout_grid,key='--Frame_grid')],
                   [sg.Frame(title='Margins',layout=layout_margins,key='--Frame_margins')]]

    width_spin = {k:textSpinner(txt=' ',init_val=getattr(lines,k)['settings']['linewidth'],width=2,key='ix'+str(jx)+'-'+'++linewidth++'+k,tooltip='line width') for k in lines.keys()}
    markersize_spin = {k: textSpinner(txt=' ', init_val=getattr(lines,k)['settings']['markersize'],width=2,key='ix'+str(jx)+'-'+'++markersize++'+k,tooltip='marker size') for k in lines.keys()}
    layout_list_lines = [[sg.Checkbox(k, enable_events=True,key='ix'+str(jx)+'-'+'++plot++'+k,default=getattr(lines,k)['settings']['plot']),\
                          sg.Text(' line: ',key='--T_lstyle_'+k),sg.InputCombo(values=line_styles_list, size=(2,1), key='ix'+str(jx)+'-'+'++linestyle++'+k,tooltip='line style',default_value=getattr(lines,k)['settings']['linestyle']),\
                          sg.Text(' ',key='--T_color_'+k),sg.InputCombo(default_value=getattr(lines,k)['settings']['color'],values=colors_list, size=(12,1), key='ix'+str(jx)+'-'+'++color++'+k,tooltip='line color'),\
                          width_spin[k][0],width_spin[k][1],width_spin[k][2],\
                          sg.Text(' marker: ',key='--T_marker_'+k),sg.InputCombo(values=markers_list, size=(2,1), key='ix'+str(jx)+'-'+'++marker++'+k,tooltip='marker style',default_value=getattr(lines,k)['settings']['marker']),\
                          sg.Text(' ',key='--T_facecolor_'+k),sg.InputCombo(default_value=getattr(lines,k)['settings']['markerfacecolor'],values=colors_list, size=(12,1), key='ix'+str(jx)+'-'+'++markerfacecolor++'+k,tooltip='marker face color'),\
                          sg.Text(' ',key='--T_edgecolor_'+k),sg.InputCombo(default_value=getattr(lines,k)['settings']['markeredgecolor'],values=colors_list, size=(12,1), key='ix'+str(jx)+'-'+'++markeredgecolor++'+k,tooltip='marker edge color'),\
                          markersize_spin[k][0],markersize_spin[k][1],markersize_spin[k][2],\
                          ] for k in lines.keys()]
    layout_right = [[sg.Column([[sg.Canvas(key='--CANVAS2-')]], key='col_fig'), sg.Column([[sg.Text('Re-scale xdata:', key='--T_scaleX'),sg.Input('1',size=(6, 1), key='ix'+str(jx)+'-'+'scalex')],[sg.Text('Re-scale ydata:', key='--T_scaleY'),sg.Input('1',size=(6, 1), key='ix'+str(jx)+'-'+'scaley')]], key='col_scale')],
                    [sg.Button('Refresh figure',key='--B_'+'ix'+str(jx)+'-refresh'),sg.Button('Edit data',key='--B_'+'ix'+str(jx)+'-editdata'),sg.Button('Save figure..',key='--B_'+'ix'+str(jx)+'-savefig'),sg.Button('Load data...',key='--B_'+'ix'+str(jx)+'-loaddata')],
                     [sg.Column(layout_list_lines,size=(600,150), scrollable=True,key='--Col_list_lines')],
                    ]

    layout = [[sg.Column(layout_left),sg.Text('    ',key='--T_sep_cols'),sg.Column(layout_right,element_justification='center')],
              [sg.Button('Close',key='--B_'+'ix'+str(jx)+'-close')]]
    return layout

def simple_layout(lines):
    print('simple_layout')
    layout = [[sg.Canvas(key='--CANVAS2-')],
              [sg.Button('Edit Figure...',key='--B_edit_fig')]]
    return layout

def popup_layout(tabs):
    tabs_layout={'Data '+k: [[sg.Multiline(default_text=getattr(tabs,k)['multiline'], size=(30,10),key='||xy||'+k)],[sg.Button('Remove tab',key='--B_remove_'+k+'_w2')]] for k in tabs.keys()}
    tab_lst_layout = [sg.Tab('Data '+k, tabs_layout['Data '+k],key='--Tab_data_'+k) for k in tabs.keys()]
    return [[sg.TabGroup([tab_lst_layout],key='--TabGroup_xy')],
             [sg.Button('Add a tab',key='--B_addtab_w2'),sg.Button('OK',key='--B_ok_w2')]]
#
def plot_fig(saved_ins,lines,show=False):
    print('plt_fig show',show)

    fig = plt.figure()

    fig.set_figwidth(float(saved_ins['figwidth']))
    fig.set_figheight(float(saved_ins['figheight']))

    ax=fig.add_subplot(111)
    for key in lines.keys():
        linei = getattr(lines,key)
        if linei['settings']['plot']:
            ls=linei['settings']

            ax.plot(linei['x']*float(saved_ins['scalex']),linei['y']*float(saved_ins['scaley']),label ='' if linei['label']=='XX' else linei['label'], linestyle=ls['linestyle'],color=ls['color'],marker=ls['marker'],markerfacecolor=ls['markerfacecolor'],
                    markeredgecolor=ls['markeredgecolor'],linewidth=ls['linewidth'],markersize=ls['markersize'])

    if saved_ins['use_title']:
        ax.set_title(saved_ins['title'],x=float(saved_ins['titlexpos']),y=float(saved_ins['titleypos']),fontsize=saved_ins['titlesize'])

    if saved_ins['use_xlabel']:
        ax.set_xlabel(saved_ins['xlabel'],fontsize=saved_ins['labelxsize'])
    if saved_ins['use_ylabel']:
        ax.set_ylabel(saved_ins['ylabel'],fontsize=saved_ins['labelysize'])
    if not saved_ins['auto_xlim']:
        ax.set_xlim(left=float(saved_ins['limxmin']),right=float(saved_ins['limxmax']),auto=not saved_ins['auto_xlim'])
    if not saved_ins['auto_ylim']:
        ax.set_ylim(bottom=float(saved_ins['limymin']),top=float(saved_ins['limymax']),auto=not saved_ins['auto_ylim'])
    if saved_ins['use_legend']:
        ax.legend(loc=saved_ins['legend_loc'],ncol=int(saved_ins['legendncol']),fontsize=float(saved_ins['legendfontsize']), handlelength=2,columnspacing=0.5)#,labelspacing=0.3,columnspacing=0.5,handletextpad=0.2)
    if saved_ins['use_grid']:
        ax.grid()
    fig.subplots_adjust(left=float(saved_ins['lmargin']), bottom=float(saved_ins['bmargin']), right=float(saved_ins['rmargin']), top=float(saved_ins['tmargin']), wspace=0, hspace=0)

    if show:
        plt.show()
    return fig
#
def draw_figure(canvas, figure):
    figure_canvas_agg = FigureCanvasTkAgg(figure, canvas)
    figure_canvas_agg.draw()
    figure_canvas_agg.get_tk_widget().pack(side='top', fill='both', expand=1)
    return figure_canvas_agg
#
def multiline2arr(xy_values):
    lst = []
    vals_lst = (xy_values+'\n').replace('\n\n\n','\n').replace('\n\n','\n').split('\n')
    vals_lst = np.array(vals_lst)[np.where(''!=np.array(vals_lst))]
    ixi,ixf=0,-1
    label=''
    if '#' in vals_lst[0]:
        ixi=1
        label=vals_lst[0].replace('\t',' ').split()[1:]
        remove_rec(label,'')
    if '' == vals_lst[-1]:
        ixf=-1
    else:
        ixf=len(vals_lst)
    for li in vals_lst[ixi:ixf]:
        newli = li.replace('\t',' ').split()
        remove_rec(newli,'')
        lst.append([float(lii) for lii in newli])
    return np.array(lst), label
#
def remove_rec(list4removal,elmnt_2_remove):
    try:
        list4removal.remove(elmnt_2_remove)
    except:
        return
    remove_rec(list4removal,elmnt_2_remove)
#
def values_from_tabs(tabs):
    tabsids = list(tabs.keys())
    labels = [np.array(multiline2arr(getattr(tabs,k)['multiline'])[1]) for k in tabsids]
    values = [np.array(multiline2arr(getattr(tabs,k)['multiline'])[0]) for k in tabsids]
    xs=[]
    ys=[]
    ls=[]
    fromtabix=[]
    i,j=-1,-1
    for l,v,t in zip(labels,values,tabsids):
        i+=1
        if 0==np.size(v):
            v=np.array([v])
        #print(l,v)
        for li,vi in zip(l,v[:,1:].T):
            j+=1
            xs.append(v[:,0])
            ys.append(vi)
            ls.append(li)
            fromtabix.append(t)
    values = {'from_tab':fromtabix, 'labels':ls,'xs': xs, 'ys':ys}
    return values

def increment_b_updn(window,event,values,incr,rdval):
    udstr = '_'+event.replace('_',' ').split()[-1]
    if udstr=='_UP':posneg=1
    elif udstr=='_DN':posneg=-1
    keywd = event.replace('--B_','').replace(udstr,'')
    counter = float(values[keywd])
    counter += posneg*incr
    window[keywd].update(round(counter,rdval))



import debyetools.tpropsgui.plotter_class as plotter
import numpy as np
import re
def find_rec_in_str(all_lines,pattrn):
    line2=str(all_lines)
    n=0
    ns=[]
    while len(all_lines)>0:
        if all_lines.find(pattrn)<0:
            break
        n = all_lines.find(pattrn)
        if len(ns)>=1:
            ns.append(ns[-1]+n+len(pattrn))
        else:
            ns.append(n)
        all_lines=all_lines[n+len(pattrn):]

    return ns

def pop_window(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx,show=False):
    print('pop_window show',show)
    tabs = plotter.tabs(initial_tabs_multilinetxt)
    data4plot = plotter.dataplot(jx)
    data4plot.load_tab_data(tabs)
    data4plot.load_lines_settings(initial_lines_settings)

    data4plot.create_window()
    data4plot.update_window(initial_fig_settings)
    data4plot.create_canvas(show=show)
    # while True:
    #     event, values = data4plot.window.read()
    #     print('plot_event:',event)
    #     if event in (plotter.sg.WIN_CLOSED, '--B_close'):
    #         break
    #
    #     elif event in ['--B_figwidth_UP','--B_figheight_UP','--B_figwidth_DN','--B_figheight_DN']:
    #         data4plot.increment_b_updn(event, values,0.1,1)
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #     elif event in ['--B_titlesize_UP','--B_labelxsize_UP','--B_labelysize_UP','--B_titlesize_DN','--B_labelxsize_DN','--B_labelysize_DN','--B_legendncol_UP','--B_legendncol_DN','--B_legendfontsize_UP','--B_legendfontsize_DN']:
    #         data4plot.increment_b_updn(event, values,1,None)
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #     elif event in ['--B_lmargin_UP','--B_rmargin_UP','--B_bmargin_UP','--B_tmargin_UP','--B_lmargin_DN','--B_rmargin_DN','--B_bmargin_DN','--B_tmargin_DN','--B_titlexpos_UP','--B_titlexpos_DN','--B_titleypos_UP','--B_titleypos_DN']:
    #         data4plot.increment_b_updn(event, values,0.01,2)
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #     elif event in ['--B_limxmax_UP','--B_limxmin_UP','--B_limxmax_DN','--B_limxmin_DN','--B_limymax_UP','--B_limymin_UP','--B_limymax_DN','--B_limymin_DN']:
    #         data4plot.increment_b_updn(event, values,.1,2)
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #     elif event in ['use_grid','use_legend','use_xlabel','use_ylabel','use_title','auto_xlim','auto_ylim']:
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #
    #     elif ('++plot++' in event or '--B_refresh' in event):
    #         data4plot.update_lines_settings()
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #
    #     elif ('++linewidth++'in event or '++markersize++'in event) :
    #         data4plot.increment_b_updn(event, values,1,None)
    #         data4plot.update_lines_settings()
    #         data4plot.update_formats()
    #         data4plot.update_canvas(show=show)
    #     elif event == '--B_loaddata':
    #         filename = plotter.sg.popup_get_file('Load Figure...')
    #         print(filename)
    #         with open(filename,'r') as f:
    #             lines=f.readlines()
    #             setting_dict = {}
    #             for line in lines:
    #                 if '->' in line[0:2] and '<-' in line[-3:-1]:
    #                     setting_dict[line[2:-3]]={}
    #                     last_keylvl0 = line[2:-3]
    #                     lvl = 0
    #                     continue
    #                 elif '*>' in line[0:2] and '<*' in line[-3:-1]:
    #                     setting_dict[last_keylvl0][line[2:-3]]={}
    #                     last_keylvl1 = line[2:-3]
    #                     lvl = 1
    #                     continue
    #                 if '*k>' in line[0:3]:
    #                     if lvl == 0:
    #                         ix0 = line.find('*k>')
    #                         ix1 = line.find('<k*')
    #                         jx0 = line.find('>|')
    #                         jx1 = line.find('|<')
    #                         val = line[jx0+2:jx1].replace('||newline','\n')
    #                         if val == 'True':
    #                             val=True
    #                         if val == 'False':
    #                             val=False
    #                         setting_dict[last_keylvl0][line[ix0+3:ix1]] = val
    #                     if lvl == 1:
    #                         ix0 = line.find('*k>')
    #                         ix1 = line.find('<k*')
    #                         jx0 = line.find('>|')
    #                         jx1 = line.find('|<')
    #                         val = line[jx0+2:jx1].replace('||newline','\n')
    #                         if val == 'True':
    #                             val=True
    #                         if val == 'False':
    #                             val=False
    #                         setting_dict[last_keylvl0][last_keylvl1][line[ix0+3:ix1]] = val
    #
    #         data4plot.window.close()
    #         tabs = plotter.tabs(setting_dict['tabs_multilinetxt'])
    #         data4plot = plotter.dataplot()
    #         data4plot.load_tab_data(tabs)
    #         data4plot.load_lines_settings(setting_dict['lines_settings'])
    #
    #         data4plot.create_window()
    #         data4plot.fig_settings=setting_dict['figure_settings']
    #         data4plot.update_window(setting_dict['figure_settings'])
    #         data4plot.create_canvas(show=show)
    #
    #     elif event == '--B_savefig':
    #
    #         filename = plotter.sg.popup_get_file('Save Figure',save_as=True)
    #         data4plot.fig.savefig(filename+".pdf")
    #
    #         with open(filename+'.ftg','w') as f:
    #             f.write('->figure_settings<-\n')
    #             for k in data4plot.fig_settings.keys():
    #                 f.write('*k>'+k+'<k*'+">|"+str(data4plot.window[k].get())+'|<\n')
    #
    #             f.write('->lines_settings<-\n')
    #             for l in data4plot.lines.keys_list:
    #                 f.write('*>'+l+'<*\n')
    #                 for k in getattr(data4plot.lines,l)['settings'].keys():
    #                     f.write('*k>'+k+'<k*'+">|"+str(getattr(data4plot.lines,l)['settings'][k])+'|<\n')
    #
    #             f.write('->tabs_multilinetxt<-\n')
    #             for i,l in enumerate(data4plot.lines.keys_list):
    #                 f.write('*>t'+str(i)+'<*\n')
    #                 f.write('*k>'+'multiline<k*>|')
    #                 f.write('#X '+getattr(data4plot.lines,l)['label']+'||newline')
    #                 sm_array = np.array([getattr(data4plot.lines,l)['x'],getattr(data4plot.lines,l)['y']]).T
    #                 for line in sm_array:
    #                     f.write(str(line[0])+' '+str(line[1])+'||newline')
    #                 f.write('|<\n')
    #
    #     elif '--B_editdata' == event:
    #         data4plot.create_popupwindow()
    #
    #         while True:
    #             event2, values2 = data4plot.popup_window.read()
    #             print(event2)
    #             if event2 in ('--B_ok_w2',plotter.sg.WIN_CLOSED):
    #                 print('ok')
    #                 break
    #
    #             if event2 == '--B_addtab_w2':
    #                 data4plot.copy_multiline2dic(add=True)
    #                 data4plot.popup_window.close()
    #                 data4plot.create_popupwindow()
    #                 data4plot.popup_window['--Tab_data_'+data4plot.tabs.keys()[-1]].select()
    #                 # print(data4plot.tabs.keys())
    #
    #             if '--B_remove_t' in event2:
    #                 #print(event2.replace('_',' ').split()[2])
    #                 delattr(data4plot.tabs,event2.replace('_',' ').split()[2])
    #                 data4plot.copy_multiline2dic()
    #                 data4plot.popup_window.close()
    #                 data4plot.create_popupwindow()
    #
    #         data4plot.copy_multiline2dic()
    #         data4plot.load_tab_data(data4plot.tabs)
    #
    #         data4plot.load_lines_settings(data4plot.lines_settings)
    #
    #         data4plot.window.close()
    #         data4plot.create_window()
    #         data4plot.update_window(data4plot.fig_settings)
    #         data4plot.create_canvas(show=show)
    #
    #         data4plot.popup_window.close()
    return data4plot

def pop_window_simple(initial_tabs_multilinetxt,initial_lines_settings,initial_fig_settings,jx,show=False):
    print('pop_window_simple show',show)
    tabs = plotter.tabs(initial_tabs_multilinetxt)
    data4plot = plotter.dataplot(jx)
    data4plot.load_tab_data(tabs)
    data4plot.load_lines_settings(initial_lines_settings)

    data4plot.create_window(simple=True)
    data4plot.fig_settings=initial_fig_settings
    data4plot.create_canvas(show=show)

    return data4plot.window
class fig:
    def __init__(self,xlabel, ylabel):
        self.colors = 100*['mediumpurple','gray','deepskyblue', 'purple','orchid','pink','aqua','cornflowerblue', 'C0','mediumpurple','gray','deepskyblue', 'purple','orchid','pink','aqua','cornflowerblue', 'C0','mediumpurple','gray','deepskyblue', 'purple','orchid','pink','aqua','cornflowerblue', 'C0','mediumpurple','gray','deepskyblue', 'purple','orchid','pink','aqua','cornflowerblue', 'C0']
        self.markers = 100*['s','+','x','o','^','>','1','<','2','s','+','x','o','^','>','1','<','2','s','+','x','o','^','>','1','<','2','s','+','x','o','^','>','1','<','2']
        self.initial_lines_settings = {}
        self.initial_fig_settings = {'figwidth':6,'figheight':4.5,'use_title':False,'title':'','titlexpos':.7,'titleypos':.9,
                                'titlesize':12,'use_xlabel':True,'use_ylabel':True,'xlabel':xlabel,'ylabel':ylabel,'labelxsize':12,
                                'labelysize':12,'auto_xlim':True,'auto_ylim':True,'limxmin':-0.5,'limxmax':110,'limymin':-1,'limymax':2,'use_legend':True,'legend_loc':'best',
                                'legendncol':1,'legendfontsize':10,'use_grid':True,'lmargin':0.11,'rmargin':0.98,'tmargin':0.95,'bmargin':0.12, 'scalex':1,'scaley':1}

        self.tabs = []
        self.types = []
        self.len = 0
    def add_set(self, X, Y, label='Y', type='line',lcolor=False, mcolor=False, mtype=False):
        tab3_str = '#T '+label+'\n'
        for Ti, Ci in zip(X, Y):
            tab3_str = tab3_str + '%.10e   %.10e  '%(Ti,Ci)+'\n'
        self.types.append(type)
        self.tabs.append(tab3_str)
        if type == 'line':
            linestyle = '-'
            mtype = 'None'
        elif type == 'dash':
            linestyle = '--'
            mtype = 'None'

        else:
            linestyle = 'None'
            if not mtype:
                mtype = self.markers[self.len]
        if not lcolor:
            lcolor = self.colors[self.len]
        if not mcolor:
            mcolor = self.colors[self.len]
        self.initial_lines_settings['l'+str(self.len)] = {'plot':True,'label':0,'linestyle':linestyle,'color':lcolor,        'marker':mtype,   'markerfacecolor':'None', 'markeredgecolor':mcolor,'linewidth':2,'markersize':10}
        self.len+=1
    def plot(self,jx,show=False):
        print('plot show',show)
        self.initial_tabs_multilinetxt = {'t'+str(i):{'multiline':self.tabs[i]} for i in range(len(self.tabs))}
        pop_window_simple(self.initial_tabs_multilinetxt,self.initial_lines_settings,self.initial_fig_settings,jx,show=show)



from PySide6.QtWidgets import QMainWindow

from debyetools.tpropsgui.ui_interatomic_params import Ui_Form as Ui_iparams

from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import numpy as np

import debyetools.tpropsgui.get_functions as get
from debyetools.tpropsgui.atomtools import atomic_color, atomic_radii, atomsPositions

def print_to_box(ptedit, txt=''):
    txt = ptedit.toPlainText()+txt
    ptedit.setPlainText(txt)
    ptedit.verticalScrollBar().setValue(ptedit.verticalScrollBar().maximum())

class windowInteratormic(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_iparams()
        self.ui.setupUi(self)

        self.cellparamsTable = self.ui.tableWidget
        self.basisTable = self.ui.tableWidget_2

        self.fig = Figure(figsize=(3, 3))
        self.canvas = FigureCanvas(self.fig)
        self.canvas_configuration()

        llayout = self.ui.horizontalLayout
        llayout.addWidget(self.canvas, 88)

        self.ui.pushButton.clicked.connect(self.on_pushButton_create_cell_clicked)
        self.ui.OKbutton.clicked.connect(self.close)

    def canvas_configuration(self):
        self.fig.set_canvas(self.canvas)

        self.ax = []
        for i in range(4):
            self.ax.append(self.canvas.figure.add_subplot(2, 2, i+1))

        for axi in self.ax:
            axi.set_axis_off()

        self.fig.subplots_adjust(hspace=0.05, right=0.99, top=.99, bottom=.01, left=0.01, wspace=0.05)

    def on_pushButton_create_cell_clicked(self):
        self.molecule.cell = get.cell(self)
        self.molecule.basis = get.basis(self)
        self.molecule.update_fomula(get.formula(self))

        self.plot_cell()


        cutoff = int(self.ui.lineEdit.text())
        self.molecule.run_pa(cutoff)
#        print('bkp1')

        ds, ns, cts = self.molecule.distances, self.molecule.num_bonds_per_formula, self.molecule.combs_types

        self.ui.plainTextEdit.setPlainText('')
        print_to_box(self.ui.plainTextEdit,'Pair analysis:\n')
        print_to_box(self.ui.plainTextEdit,'distances  | # of pairs per type\n')
        print_to_box(self.ui.plainTextEdit,'           | ' + '  '.join(['%s' for _ in cts])%tuple(cts)+'\n')
        for d, n in zip(ds, ns):
            print_to_box(self.ui.plainTextEdit,'%.6f  '%(d)+' | ' + ' '.join(['%.2f' for _ in n])%tuple(n)+'\n')

        a, b, c = 0.5, 0.5, -len(cts)
        ntypes =(-b+np.sqrt(b**2-4*a*c))/(2*a)
        if self.eos_str == 'MP':
            params = [0.35, 1, 3.2]*len(cts)
        elif self.eos_str == 'EAM':
            params = [3.65e-03, 1.24e-02, 2.68e-04, 1.03e-02,
                        1.49e-01, 5.22e-02]*len(cts)+[2.26e+00,
                        6.61e-02, 3.01e-01, 5.31e-05]*int(ntypes)

        params = ', '.join([str(pi) for pi in params])
        self.params_interatomic = ', '.join([params for _ in range(len(cts))])

        self.ui.lineEdit_3.setText(self.params_interatomic)

        self.ui.OKbutton.setEnabled(True)

    def plot_atom_xy(self, ax, atom_position, atom_type):
        x, y, z = atom_position
        ax.plot(x, y, marker='o', mfc=atomic_color[atom_type], mec='black', ms=10*atomic_radii[atom_type])
    def plot_atom_xz(self, ax, atom_position, atom_type):
        x, y, z = atom_position
        ax.plot(x, -z, marker='o', mfc=atomic_color[atom_type], mec='black', ms=10*atomic_radii[atom_type])
    def plot_atom_yz(self, ax, atom_position, atom_type):
        x, y, z = atom_position
        ax.plot(-z, y, marker='o', mfc=atomic_color[atom_type], mec='black', ms=10*atomic_radii[atom_type])

    def plot_cell_xy(self, ax, cell):
        x, y = self.gen_cell_points_xy(cell)
        ax.plot(x, y, linestyle='--', color='gray', linewidth=1)
    def plot_cell_xz(self, ax, cell):
        x, y = self.gen_cell_points_xz(cell)
        ax.plot(x, -1*np.array(y), linestyle='--', color='gray', linewidth=1)
    def plot_cell_yz(self, ax, cell):
        y, z = self.gen_cell_points_yz(cell)
        ax.plot(-1*np.array(z), y, linestyle='--', color='gray', linewidth=1)

    def gen_cell_points_xy(self, cell):
        xbox1 = [
         0, #1
         cell[0, 0], #2
         cell[0, 0] + cell[1, 0], #5
         cell[1, 0], #3
         0, #1
         ]
        ybox1 = [
        0, #1
        cell[0, 1], #2
        cell[0, 1] + cell[1, 1], #5
        cell[1, 1], #3
        0, #1
        ]

        return xbox1, ybox1

    def gen_cell_points_xz(self, cell):
        xbox1 = [
         0, #1
         cell[0, 0], #2
         cell[0, 0] + cell[2, 0], #6
         cell[2, 0], #4
         0, #1
        ]
        zbox1 = [
         0, #1
         cell[0, 2], #2
         cell[0, 2] + cell[2, 2], #6
         cell[2, 2], #4
         0, #1
        ]
        return xbox1, zbox1

    def gen_cell_points_yz(self, cell):
        ybox1 = [
         0, #1
         cell[1, 1], #3
         cell[1, 1] + cell[2, 1], #7
         cell[2, 1], #4
         0, #1
        ]
        zbox1 = [
         0, #1
         cell[1, 2], #3
         cell[1, 2] + cell[2, 2], #7
         cell[2, 2], #4
         0, #1
        ]

        return ybox1, zbox1

    def plot_cell(self):
        for axi in self.ax:
            axi.cla()
            axi.set_axis_off()

        atoms = atomsPositions(self.molecule.formula, self.molecule.cell, self.molecule.basis)

        for ai in atoms:
            self.plot_atom_xz(self.ax[3], ai.position, ai.type)
        self.plot_cell_xz(self.ax[3], self.molecule.cell)
        for ai in atoms:
            self.plot_atom_xy(self.ax[1], ai.position, ai.type)
        self.plot_cell_xy(self.ax[1], self.molecule.cell)

        for ai in atoms:
            self.plot_atom_yz(self.ax[0], ai.position, ai.type)
        self.plot_cell_yz(self.ax[0], self.molecule.cell)


        self.ax[1].plot([-0.5,0.5],[-0.5,-0.5], 'b', linewidth=1)
        self.ax[1].plot([-0.5,-0.5],[-0.5,0.5], 'r', linewidth=1)

        self.ax[0].plot([0.5,0.5],[-0.5,0.5], 'r', linewidth=1)
        self.ax[0].plot([0.5,-0.5],[-0.5,-0.5], 'g', linewidth=1)

        self.ax[3].plot([-0.5,0.5],[0.5,0.5], 'b', linewidth=1)
        self.ax[3].plot([-0.5,-0.5],[0.5,-0.5], 'g', linewidth=1)

        self.canvas.draw()

    def closeEvent(self, event):
        self.molecule.number_of_NNs = int(self.ui.lineEdit_2.text())
        self.args = self.molecule.formula, self.molecule.cell, self.molecule.basis, self.molecule.cutoff, self.molecule.number_of_NNs
        self.external_params_lineText.setText(self.params_interatomic)
        self.ui.lineEdit_3.setText('')
        event.accept()
